<?php

use \Moreflo\Order;

class MfAdmin
{

	private $api;
	private $doing_payment = false;

    public function __construct()
    {

	    $this->api = new \MfApi();

    	add_action('admin_menu', array($this, 'create_plugin_admin'));

	    $this->set_hook_save_article( true );
	    $this->set_hook_save_order( true );

	    add_action("woocommerce_new_order", array($this, 'order_needs_receipt'), 5, 2);

	    add_action("woocommerce_new_order", array($this, 'save_order'), 10, 2);
		add_action("mf_retry_save_order", array($this, 'retry_save_order'), 10, 1);
		/**
		 * Stock reduce call no need as per v3, When we passing order paymentstatus as "FullyPaid" It automatically reduce the stock,
		 * So remove complete order hook 09-17-2020
		 */
		//add_action("woocommerce_order_status_changed", array($this,'reset_stock_quan'), 10, 3);

	    add_action("woocommerce_create_refund", array($this, 'validate_refund_order'), 10, 2);
	    add_action("woocommerce_refund_created", array($this, 'refund_order'), 10, 2);

	    add_action("woocommerce_pre_payment_complete", array($this, 'pre_payment_complete'));
	    add_action("woocommerce_payment_complete", array($this, 'payment_complete'));

	    add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_scripts' ) );

	    register_activation_hook( 'moreflo-sync/wp_moreflo.php', array($this, 'activate') );
	    register_deactivation_hook( 'moreflo-sync/wp_moreflo.php', array($this, 'deactivate') );

	    add_action (MF_SYNC_ARTICLES, array($this, 'do_mf_sync_articles'));
		add_action (MF_SYNC_ARTICLES, array($this, 'do_mf_sync_groups'), 5); // Must be ran before sync articles
		add_action (MF_SYNC_ARTICLES, array($this, 'do_mf_sync_campaigns'), 20);
		add_action (MF_SYNC_ARTICLES, array($this, 'do_mf_sync_campaigns_article_sets'), 21);
		add_action (MF_SYNC_ARTICLES, array($this, 'do_mf_sync_bookkeeping'), 22);
	    add_action ('mf_sync_orders', array($this, 'do_mf_sync_orders_latest'));
	    add_filter('cron_schedules', array( $this, 'add_custom_sync_times' ) );

		add_action("wp_ajax_mf_sync", array($this, 'ajax_sync'));
		add_action("wp_ajax_mf_tax_sync", array($this, 'ajax_tax_rules'));
		add_action("wp_ajax_mf_ajax_groups", array($this, 'ajax_groups'));
		add_action("wp_ajax_mf_ajax_clean_groups", array($this, 'ajax_clean_groups'));
		add_action("wp_ajax_mf_ajax_campaigns", array($this, 'ajax_campaigns'));
		add_action("wp_ajax_mf_ajax_clean_campaigns", array($this, 'ajax_clean_campaigns'));
		add_action("wp_ajax_mf_ajax_campaigns_article_sets", array($this, 'ajax_campaigns_article_sets'));
		add_action("wp_ajax_mf_ajax_bookkeeping", array($this, 'ajax_bookkeeping'));
	    add_action("wp_ajax_nopriv_mf_sync", array($this, 'wp_ajax_nopriv_mf_sync'));

	    add_action("wp_ajax_mf_ajax_reactivate_cron", array($this, 'mf_ajax_reactivate_cron'));

		// Show Apps order number in admin
		add_filter( 'manage_shop_order_posts_columns', [ $this, 'manage_shop_order_posts_columns' ], 20 );
		add_filter( 'manage_shop_order_posts_custom_column', [ $this, 'manage_shop_order_posts_custom_column' ], 10, 2 );
    }

	function activate() {
	}

	function deactivate()
	{
		wp_clear_scheduled_hook(MF_SYNC_ARTICLES);
		// delete_option(MOREFLO_API_URL);
		delete_option('mf_account_ids');
		delete_option('mf_article_sync');
		delete_option(MF_ARTICLE_SYNC_LAST_UPDATED);
	}

	function set_hook_save_article( $on ) {
		$api_options = get_option(MOREFLO_API_URL);

		if (
			$on &&
			! (
				! empty( $api_options['sync_only_product_stock_amount'] ) &&
				$api_options['sync_only_product_stock_amount'] == 'enabled'
			)
		) {
			add_action( 'woocommerce_new_product',  array($this, 'save_product'), 10, 2 );
			add_action( 'woocommerce_update_product', array($this, 'save_product'), 10, 2 );
		} else {
			remove_action( 'woocommerce_new_product',  array($this, 'save_product'), 10 );
			remove_action( 'woocommerce_update_product', array($this, 'save_product'), 10 );
		}
	}

	function set_hook_save_order( $on ) {
		if ( $on ) {
			add_action( 'woocommerce_update_order', array($this, 'save_order'), 10, 2 );
		} else {
			remove_action( 'woocommerce_update_order', array($this, 'save_order'), 10, 2 );
		}
	}

	function pre_payment_complete() {
		$this->doing_payment = true;
	}

	function payment_complete() {
		$this->doing_payment = false;
	}

	function order_needs_receipt($order_id, $order = null) {
		$api_options = get_option(MOREFLO_API_URL);

		// Is receipts enabled?
		if ( ! isset( $api_options['is_receipt_active'] ) ) {
			return;
		}
		
		if ( 'enabled' != $api_options['is_receipt_active'] ) {
			return;
		}

		// If this order lacks receipt, set status
		if (!get_post_meta($order_id, '_mf_receipt_status', true)) {
			update_post_meta($order_id, '_mf_receipt_status', 'needs-receipt');
		}
	}

	function retry_save_order( $order_id ) {
		$order = wc_get_order( $order_id );
		if ( $order ) {
			$order->add_order_note( esc_html__( 'Retrying order sync.', 'wpmf' ) );
			$this->save_order( $order_id, $order );
		}
	}

	function save_order($order_id, $order = null) {
		$api_options = get_option(MOREFLO_API_URL);
		$sync_orders = ! isset( $api_options['sync_new_orders_to_apps'] ) || 'disabled' != $api_options['sync_new_orders_to_apps'];

        $lock = new Moreflo_Lock( __FUNCTION__ .'-'. $order_id );
        if ( ! $lock->lock() ) {
			// Save order is already running for this order, retry later
			wp_schedule_single_event(time() + 60, 'mf_retry_save_order', [$order_id]);
			$order->add_order_note( esc_html__( 'Aborted syncing order. It is already running.', 'wpmf' ) );
            return;
        }

		if($order instanceof WC_Order) {
			$order_instance = new Order($order);
			$mf_order = $order_instance->order_to_moreflo_order();
			$mf_order_type = $order_instance->get_order_type();

			$invalidOrders = get_post_meta($order_id, 'multiple_bookable_orders', true);
			$mf_order_total = $order_instance->orderTotal();

			if(!empty($mf_order) && $mf_order_type != 'INVALID_ORDER' && empty($invalidOrders)) {

				if($mf_order_total > 0){
					$old_hash = get_post_meta($order_id, MF_HASH, true);
					$new_hash = generate_order_hash($mf_order);

					if($old_hash != $new_hash) {
						$response = false;
						if ( $sync_orders ) {
							$action = $old_hash ? 'update' : 'add';
							if($mf_order_type == 'SERVICE_ORDER') {
								$response = $this->api->post_service_order($mf_order);
								$order_instance->validate_moreflo_order_response(SERVICE, $action, $response);
							} else {
								$mf_order = apply_filters( 'moreflo_sync_before_order_sync', $mf_order, $order, $action );
								$response = $this->api->post_order($mf_order);
								$order_instance->validate_moreflo_order_response(NORMAL, $action, $response);
							}
						}

						$mf_receipt = $order_instance->order_to_moreflo_receipt();
						if ( is_wp_error( $mf_receipt ) ) {
							// Failure
							update_post_meta($order_id, '_mf_receipt_status', 'receipt-failed');

							MfLog::log(DEBUG, 'Order receipt failure. (Order ID: '. $order_id .'; Message: '. $mf_receipt->get_error_message() .')');
						} elseif ( $mf_receipt ) {
							// We can safely send receipt
							$receipt_response = $this->api->post_receipt( $mf_receipt );
							update_post_meta($order_id, 'mf_receipt_response', $receipt_response);
							if ( ! $receipt_response ) {
								// Failure
								update_post_meta($order_id, '_mf_receipt_status', 'receipt-failed');
								MfLog::log(DEBUG, 'Order receipt failed. (Order ID: '. $order_id .')');
								
								$order->add_order_note( 'Order receipt failed.' );
							} else {
								update_post_meta($order_id, '_mf_receipt_status', 'receipt-sent');
								MfLog::log(DEBUG, 'Order receipt sent successfully. (Order ID: '. $order_id .')');
								$receipt_response_obj = json_decode( $receipt_response );
								// Success
								$receipt_id = $receipt_response_obj->Receipt->Id;
								$voucher_id = $receipt_response_obj->Receipt->Voucher->Id;
								update_post_meta( $order_id, 'mf_receipt_id', $receipt_id );
								update_post_meta( $order_id, 'mf_receipt_voucher_id', $voucher_id );
								$order->add_order_note( sprintf( esc_html__( 'Order receipt sent successfully. (Receipt: %s; Voucher: %s)', 'wpmf' ), $receipt_id, $voucher_id ) );
							}
						}

						if(!empty($mf_order['BookingStartTime']) && !empty($mf_order['BookingEndTime']))
						{
							$bookableOrders['StartTime'] = $mf_order['BookingStartTime'];
							$bookableOrders['StopTime'] = $mf_order['BookingEndTime'];

							$mapper = new MfMapper();
							$mf_order_infor = $mapper->transforms_order_data($bookableOrders, 'StartTime', 'StopTime');					
							$new_appoint_hash = generate_order_hash($mf_order_infor);
							update_post_meta($order_id, MF_ORDER_INFOR_HASH, $new_appoint_hash);
						}
						

						if($response) {
							update_post_meta($order_id, MF_HASH, $new_hash);
							update_post_meta($order_id, MF_SYNC_STATUS, __('Synced', 'wpmf'));
							update_post_meta($order_id, 'mf_last_sync_response', addslashes($response));

							/**
							 * Only update order item stock if not doing payment. Otherwise, it will be updated when payment is completed.
							 *      WC_Order->payment_complete() is called when payment is completed.
							 *      This triggers WC_Order->save(), which triggers WC_Order->status_transition(), which triggers action woocommerce_order_status_processing
							 *      Action triggers wc_maybe_reduce_stock_levels(), which won't see that articles have been synced, and further decreases stock.
							 */
							if ( ! $this->doing_payment ) {
								$this->reset_stock_quan($order_id);
							} else {
								MfLog::log( DEBUG, 'Skipping stock update because doing payment (Order ID: ' . $order_id . ')' );
							}

							MfLog::log(DEBUG, 'Order pushed and updated');
						}
					} else {
						MfLog::log(DEBUG, 'Hash verification failed');
					}
				}else{
					MfLog::log(DEBUG, 'Order total is less than zero. (Order ID: '. $order_id .')');	
				}
			} else {
				$order->add_order_note(__('This order cannot be pushed to the MoreFlo since it\'s contains the booking type product(s).', 'wpmf'));
				update_post_meta($order_id, 'multiple_bookable_orders', true);
				MfLog::log(DEBUG, 'Order pushing failed since it\'s contains the bookable products. (Order ID: '. $order_id .')');
			}
		}

        $lock->clear_lock();

		return false;
	}

	public function reset_stock_quan($order_id)
	{
		MfLog::log(DEBUG, 'Start resetting product stocks');
		$includeTypes = [ 'simple', 'variable', 'grouped'];
		$excludeStatus = [ 'failed', 'refunded', 'cancelled', 'completed'];
		$order = wc_get_order( $order_id );
		$orderStatus = $order->get_status();
		if($order instanceof WC_Order && !in_array($orderStatus, $excludeStatus)) {
		$order_instance = new Order($order);
			foreach ( $order->get_items() as $item ) {
				if(!empty($item->get_product_id())){
					$product = wc_get_product($item->get_product_id());
					$ArticleNo = $product->get_sku();
					if(in_array($product->get_type(), $includeTypes) && !empty($ArticleNo)){
						$order_instance->updateArticleQunatity($ArticleNo);
					}
				}
			}
		}
		else
		{
			MfLog::log(DEBUG, 'Order is not a woo order/Order status is excluding order status #'.$orderStatus.' OrderID#'.$order_id);
		}
		MfLog::log(DEBUG, 'End resetting product stocks');
	}

	/**
	 * Checks if refund can be made in a way that is valid for Apps.
	 * 
	 * @param int $order_id
	 * @param array $args
	 * 
	 * @throws Exception if refund is not valid
	 */
	function validate_refund_order( $order_id, $args ) {
		$order_refund = wc_get_order( $order_id );
        $order_items = $order_refund->get_items();

		$quantity_order_items = array_filter( $order_items, function ( $order_item ) {
			return $order_item->get_quantity() != 0;
		} );
		$non_quantity_order_items = array_filter( $order_items, function ( $order_item ) {
			return $order_item->get_quantity() == 0;
		} );
		if ( count( $quantity_order_items ) > 0 && count( $non_quantity_order_items ) > 0 ) {
			// Since row quantity cannot be 0 we cannot have some rows with quantity and some without.
			// If stock management is off we can set the quantity to 1, but not if it's on. That would
			// mess with the stock in Apps
			throw new Exception( __( 'Refunds cannot contain both rows with quantity and rows without. Please make the refund in two steps.', 'wpmf' ) );
		}
	}

	function refund_order( $order_id, $args ) {
		$api_options = get_option(MOREFLO_API_URL);

		$order = new WC_Order( $order_id );
		$parent_id = $order->get_parent_id();
		$parent_order = new WC_Order( $parent_id );

		// Create negative order in Apps
		// Use parent order as template to get all customer info
		$parent_refund_order_instance = new Order( $parent_order );
		$parent_mf_order = $parent_refund_order_instance->order_to_moreflo_order();

		// If order is fully refunded, we will instead sync order as cancelled
		if ( did_action( 'woocommerce_order_fully_refunded' ) ) {
			// We still need to sync refund receipt
			$this->refund_order_receipt( $order_id, $parent_order, $parent_refund_order_instance, $args['restock_items'] );
			return;
		}

		// Create refund info
		$refund_order_instance = new Order( $order );
		$refund_mf_order = $refund_order_instance->refund_order_to_moreflo_order( $parent_mf_order, $parent_id );

		// Ensure don't have an empty order
		if ( empty( $refund_mf_order ) || empty( $refund_mf_order['Rows'] ) ) {
			MfLog::log( DEBUG, 'Refund order failed, empty. (order id: '. $order_id .')' );
			return;
		}

		// Post to Apps
		$refund_mf_order = apply_filters( 'moreflo_sync_before_order_refund_sync', $refund_mf_order, $order, $parent_order );
		$response = $this->api->post_order( $refund_mf_order );
		$refund_order_instance->validate_moreflo_order_response( NORMAL, 'add', $response );
		$new_hash = generate_order_hash( $refund_mf_order );

		// Check error response
		if ( ! $response ) {
			MfLog::log( DEBUG, 'Refund order failed, no response from Apps. (order id: '. $order_id .')' );
			return;
		}

		// Update last sync
		update_post_meta( $order_id, MF_HASH, $new_hash );
		update_post_meta( $order_id, MF_SYNC_STATUS, __( 'Synced', 'wpmf' ) );
		update_post_meta( $order_id, 'mf_last_sync_response', $response );
		$this->reset_stock_quan( $order_id );

		$this->refund_order_receipt( $order_id, $parent_order, $parent_refund_order_instance, $args['restock_items'] );

		MfLog::log( DEBUG, 'Refund order pushed and updated' );
	}

	/**
	 * Sends refund order receipt if activated in settings
	 * 
	 * @param int $order_id
	 * @param WC_Order $parent_order
	 * @param Moreflo\Order $parent_refund_order_instance
	 * @param bool $restock_items
	 */
	private function refund_order_receipt( $order_id, $parent_order, $parent_refund_order_instance,
		$restock_items ) {
		// Create new receipt for refunded order
		$order_refund = wc_get_order( $order_id );
		$mf_receipt = $parent_refund_order_instance->order_to_moreflo_receipt_refund( $order_refund, $restock_items );
		if ( is_wp_error( $mf_receipt ) ) {
			// Failure
			update_post_meta($order_id, '_mf_receipt_status', 'receipt-failed');

			MfLog::log(DEBUG, 'Order receipt failure. (Order ID: '. $order_id .'; Message: '. $mf_receipt->get_error_message() .')');
		} elseif ( $mf_receipt ) {
			// We can safely send receipt
			$receipt_response = $this->api->post_receipt( $mf_receipt );
			update_post_meta($order_id, 'mf_receipt_response', $receipt_response);
			if ( ! $receipt_response ) {
				// Failure
				update_post_meta($order_id, '_mf_receipt_status', 'receipt-failed');
				MfLog::log(DEBUG, 'Order receipt  refund failed. (Order ID: '. $order_id .')');
				
				$parent_order->add_order_note( 'Order receipt refund failed. Error message: '. $receipt_response->get_error_message() );
			} else {
				update_post_meta($order_id, '_mf_receipt_status', 'receipt-sent');
				MfLog::log(DEBUG, 'Order receipt refund sent successfully. (Order ID: '. $order_id .')');
				$receipt_response_obj = json_decode( $receipt_response );
				// Success
				$receipt_id = $receipt_response_obj->Receipt->Id;
				$voucher_id = $receipt_response_obj->Receipt->Voucher->Id;
				update_post_meta( $order_id, 'mf_receipt_id', $receipt_id );
				update_post_meta( $order_id, 'mf_receipt_voucher_id', $voucher_id );
				$parent_order->add_order_note( 'Order receipt refund sent successfully. (Receipt: '. $receipt_id .'; Voucher: '. $voucher_id .')' );
			}
		}
	}

	function ajax_sync() {

		if (!check_ajax_referer( 'mf_ajax_sync_nonce', 'security' )) {
			exit("Nonce failed");
		}
		$sync_start_time = mf_get_current_time();

	    $this->set_hook_save_article( false );

		$page = !empty($_POST['page']) ? $_POST['page'] : 0;
		$page_size = !empty($_POST['page_size']) ? $_POST['page_size'] : 100;

		$mapper = new MfMapper();
		$status = $mapper->pull_articles($page, $page_size);

		// Start time is used to prevent gap in sync.
		// https://northmill.atlassian.net/browse/FLOT-1163
		update_option( MF_ARTICLE_SYNC_LAST_UPDATED, $sync_start_time );

		if(!empty($_SERVER['HTTP_X_REQUESTED_WITH']) && strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest') {
			echo json_encode($status);
		}
		else {
			echo '';
		}

		die();

	}
	function ajax_tax_rules() {

		if (!check_ajax_referer( 'mf_ajax_sync_nonce', 'security' )) {
			exit("Nonce failed");
		}

		$MfSettings = new MfSettings();
		$response = $this->api->get_account_codes();

		if($response) {

			$data = json_decode($response);

			if(!empty($data) && !empty($data->Total)) {
				$status['page'] = $data->Page;
				$status['total'] = $data->Total;
			}

			if($data && isset($data->list) && is_array($data->list)) {
				delete_option('mf_account_ids');
				update_option('mf_account_ids', $data->list);
				$MfSettings->sync_tax_rules($data->list);
			}
		}

		if(!empty($_SERVER['HTTP_X_REQUESTED_WITH']) && strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest') {
			echo json_encode($status);
		}
		else {
			echo '';
		}

		die();
	}

	function ajax_groups() {
		$api_options = get_option(MOREFLO_API_URL);
		if ( ! isset( $api_options['is_product_categories_active'] )
			|| 'enabled' != $api_options['is_product_categories_active']
		) {
			echo json_encode(['total' => 1]);
			exit;
		}

		if (!check_ajax_referer( 'mf_ajax_sync_nonce', 'security' )) {
			exit("Nonce failed");
		}

		$page = !empty($_POST['page']) ? $_POST['page'] : 0;
		$page_size = !empty($_POST['page_size']) ? $_POST['page_size'] : 100;

		$syncer = new MF_Sync_Groups_Sync();
		$status = $syncer->sync($page, $page_size, false);

		if(!empty($_SERVER['HTTP_X_REQUESTED_WITH']) && strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest') {
			echo json_encode($status);
		}
		else {
			echo '';
		}

		die();

	}

	function ajax_clean_groups() {
		$api_options = get_option(MOREFLO_API_URL);
		if ( ! isset( $api_options['is_product_categories_active'] )
			|| 'enabled' != $api_options['is_product_categories_active']
		) {
			echo json_encode(['total' => 1]);
			exit;
		}

		if (!check_ajax_referer( 'mf_ajax_sync_nonce', 'security' )) {
			exit("Nonce failed");
		}

		$page = !empty($_POST['page']) ? $_POST['page'] : 0;

		$syncer = new MF_Sync_Groups_Sync();
		$status = $syncer->clean_groups($page);

		if(!empty($_SERVER['HTTP_X_REQUESTED_WITH']) && strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest') {
			echo json_encode($status);
		}
		else {
			echo '';
		}

		die();

	}

	function ajax_campaigns() {
		if ( ! moreflo_sync_campaigns_enabled() ) {
			echo json_encode(['total' => 1]);
			exit;
		}

		if (!check_ajax_referer( 'mf_ajax_sync_nonce', 'security' )) {
			exit("Nonce failed");
		}

		$page = !empty($_POST['page']) ? $_POST['page'] : 0;
		$page_size = !empty($_POST['page_size']) ? $_POST['page_size'] : 100;

		$syncer = new MF_Sync_Campaigns_Sync();
		$status = $syncer->sync($page, $page_size, false);

		if(!empty($_SERVER['HTTP_X_REQUESTED_WITH']) && strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest') {
			echo json_encode($status);
		}
		else {
			echo '';
		}

		die();
	}

	function ajax_clean_campaigns() {
		if ( ! moreflo_sync_campaigns_enabled() ) {
			echo json_encode(['total' => 1]);
			exit;
		}

		if (!check_ajax_referer( 'mf_ajax_sync_nonce', 'security' )) {
			exit("Nonce failed");
		}

		$syncer = new MF_Sync_Campaigns_Sync();
		$status = $syncer->clean_campaigns(0);

		if(!empty($_SERVER['HTTP_X_REQUESTED_WITH']) && strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest') {
			echo json_encode($status);
		}
		else {
			echo '';
		}

		die();

	}

	function ajax_campaigns_article_sets() {
		if ( ! moreflo_sync_campaigns_enabled() ) {
			echo json_encode(['total' => 1]);
			exit;
		}

		if (!check_ajax_referer( 'mf_ajax_sync_nonce', 'security' )) {
			exit("Nonce failed");
		}

		$page = !empty($_POST['page']) ? $_POST['page'] : 0;
		$page_size = !empty($_POST['page_size']) ? $_POST['page_size'] : 100;

		$syncer = new MF_Sync_Campaigns_Sync_Sets();
		$status = $syncer->sync($page, $page_size, false);

		if(!empty($_SERVER['HTTP_X_REQUESTED_WITH']) && strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest') {
			echo json_encode($status);
		}
		else {
			echo '';
		}

		die();
	}

	function ajax_bookkeeping() {
		if ( ! moreflo_sync_receipts_enabled() ) {
			echo json_encode(['total' => 1]);
			exit;
		}

		if (!check_ajax_referer( 'mf_ajax_sync_nonce', 'security' )) {
			exit("Nonce failed");
		}

		$page = !empty($_POST['page']) ? $_POST['page'] : 0;
		$page_size = !empty($_POST['page_size']) ? $_POST['page_size'] : 100;

		$syncer = new MF_Sync_Bookkeeping_Vat_Sync();
		$status = $syncer->sync($page, $page_size, false);

		if(!empty($_SERVER['HTTP_X_REQUESTED_WITH']) && strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest') {
			echo json_encode($status);
		}
		else {
			echo '';
		}

		die();
	}

	function mf_ajax_reactivate_cron() {
		if (!check_ajax_referer( 'mf_ajax_sync_nonce', 'security' )) {
			wp_send_json_error( __( 'Security check failed', 'wpmf' ) );
		}

		$mf_settings = get_option( 'moreflo_api_url' );
		if ( ! isset( $mf_settings['mf_sync_status'] ) ) {
			wp_send_json_error( __( 'Sync disabled', 'wpmf' ) );
		}
		if ( 'enabled' != $mf_settings['mf_sync_status'] ) {
			wp_send_json_error( __( 'Sync disabled', 'wpmf' ) );
		}
		if ( ! isset( $mf_settings['mf_sync_articles_interval'] ) ) {
			wp_send_json_error( __( 'Invalid sync interval', 'wpmf' ) );
		}

		wp_clear_scheduled_hook('mf_sync_articles');
		wp_clear_scheduled_hook('mf_sync_orders');

		$a = wp_schedule_event( time(), $mf_settings['mf_sync_articles_interval'], 'mf_sync_articles' );
		$b = wp_schedule_event( time(), $mf_settings['mf_sync_articles_interval'], 'mf_sync_orders' );

		if ( ! $a || is_wp_error( $a ) || ! $b || is_wp_error( $b ) ) {
			wp_send_json_error( __( 'Activation failed. Is cron disabled on your server?', 'wpmf' ) );
		}

		update_option( MF_SYNC_CRON_CHECK_EMAIL, false );

		wp_send_json_success();
	}

	function nopriv_mf_sync() {
		echo '';
		die();
	}

	function add_custom_sync_times( $array )
	{
		$array['everyminute'] = array(
			INTERVAL => 60,
			DISPLAY => __('Every minute (MoreFlo)', 'wpmf')
		);

		$array['everyfiveminutes'] = array(
			INTERVAL => 300,
			DISPLAY => __('Every 5 minutes (MoreFlo)', 'wpmf')
		);

		return $array;
	}

	function do_mf_sync_articles() {
		$sync_start_time = mf_get_current_time();
        $lock = new Moreflo_Lock( __FUNCTION__ );
        if ( ! $lock->lock() ) {
            MfLog::log( 'warning', 'Cron lock in place, skipping cron' );
            return;
        }

	    $this->set_hook_save_article( false );

		MfLog::log(DEBUG, 'Cron Started - Article Sync');

		$per_page = 100;
		$mapper = new MfMapper();
		$last_updated_at = get_option(MF_ARTICLE_SYNC_LAST_UPDATED);

		$params = array(
			'ArticleInfoUpdatedSince' => $last_updated_at,
		);

		$status = $mapper->pull_articles(0, $per_page, $params);

		if(!empty($status[TOTAL]) && $status[TOTAL] > $per_page) {
			$page = 1;
			$total_pages = $status[TOTAL] / $per_page;

			while($page < $total_pages) {
				$mapper->pull_articles($page, $per_page, $params);
				$page++;
			}
		}

		$params = array(
			'QuantityUpdatedSince' => $last_updated_at
		);

		$status = $mapper->pull_articles(0, $per_page, $params);

		if(!empty($status[TOTAL]) && $status[TOTAL] > $per_page) {
			$page = 1;
			$total_pages = $status[TOTAL] / $per_page;

			while($page < $total_pages) {
				$mapper->pull_articles($page, $per_page, $params);
				$page++;
			}
		}

		// Start time is used to prevent gap in sync.
		// https://northmill.atlassian.net/browse/FLOT-1163
		update_option( MF_ARTICLE_SYNC_LAST_UPDATED, $sync_start_time );
		
        $lock->clear_lock();

		MfLog::log(DEBUG, 'Cron Ended - Article Sync');
    }
	
	function do_mf_sync_groups() {
		if ( ! moreflo_sync_groups_enabled() ) {
			return;
		}

        $lock = new Moreflo_Lock( __FUNCTION__ );
        if ( ! $lock->lock() ) {
            MfLog::log( 'warning', 'Cron lock in place, skipping cron' );
            return;
        }

		MfLog::log('notice', 'Cron Started - Group Sync');

		$per_page = 1000;
		$syncer = new MF_Sync_Groups_Sync();

		$status = $syncer->sync(0, $per_page, false);

		if(!empty($status[TOTAL]) && $status[TOTAL] > $per_page) {
			$page = 1;
			$total_pages = $status[TOTAL] / $per_page;

			while($page < $total_pages) {
				$syncer->sync($page, $per_page, false);
				$page++;
			}
		}

		MfLog::log('notice', 'Cron Started - Clean Group');

		$syncer->clean_groups(0);

		MfLog::log('notice', 'Cron Ended - Clean Group');
		
        $lock->clear_lock();

		MfLog::log('notice', 'Cron Ended - Group Sync');
	}

	function do_mf_sync_campaigns() {
		if ( ! moreflo_sync_campaigns_enabled() ) {
			return;
		}

        $lock = new Moreflo_Lock( __FUNCTION__ );
        if ( ! $lock->lock() ) {
            MfLog::log( 'warning', 'Cron lock in place, skipping cron' );
            return;
        }

		MfLog::log('notice', 'Cron Started - Campaigns Sync');

		$per_page = 100;
		$campaigns_sync = new MF_Sync_Campaigns_Sync();

		$status = $campaigns_sync->sync(0, $per_page, false);

		if(!empty($status['total']) && $status['total'] > $per_page) {
			$page = 1;
			$total_pages = $status['total'] / $per_page;

			while($page < $total_pages) {
				$campaigns_sync->sync($page, $per_page, false);
				$page++;
			}
		}

		$campaigns_sync->clean_campaigns(0);

        $lock->clear_lock();

		MfLog::log('notice', 'Cron Ended - Campaigns Sync');
	}

	function do_mf_sync_campaigns_article_sets() {
		if ( ! moreflo_sync_campaigns_enabled() ) {
			return;
		}

        $lock = new Moreflo_Lock( __FUNCTION__ );
        if ( ! $lock->lock() ) {
            MfLog::log( 'warning', 'Cron lock in place, skipping cron' );
            return;
        }

		MfLog::log('notice', 'Cron Started - Campaigns Article Set Sync');

		$per_page = 100;
		$new_last_update_at = mf_get_current_time();
		$last_updated_at = get_option('mf_sync_campaigns_article_sets_last_updated');
		$syncer = new MF_Sync_Campaigns_Sync_Sets();

		$status = $syncer->sync(0, $per_page, $last_updated_at);

		if(!empty($status['total']) && $status['total'] > $per_page) {
			$page = 1;
			$total_pages = $status['total'] / $per_page;

			while($page < $total_pages) {
				$syncer->sync($page, $per_page, $last_updated_at);
				$page++;
			}
		}

		update_option( 'mf_sync_campaigns_article_sets_last_updated', $new_last_update_at);

        $lock->clear_lock();

		MfLog::log('notice', 'Cron Ended - Campaigns Article Set Sync');
	}

	function do_mf_sync_bookkeeping() {
		if ( ! moreflo_sync_receipts_enabled() ) {
			return;
		}

        $lock = new Moreflo_Lock( __FUNCTION__ );
        if ( ! $lock->lock() ) {
            MfLog::log( 'warning', 'Cron lock in place, skipping cron' );
            return;
        }

		MfLog::log('notice', 'Cron Started - Bookkeeping Account Sync');

		$per_page = 100;
		$new_last_update_at = mf_get_current_time();
		$last_updated_at = get_option('mf_sync_bookkeeping_last_updated');
		$syncer = new MF_Sync_Bookkeeping_Vat_Sync();

		$status = $syncer->sync(0, $per_page, $last_updated_at);

		if(!empty($status['total']) && $status['total'] > $per_page) {
			$page = 1;
			$total_pages = $status['total'] / $per_page;

			while($page < $total_pages) {
				$syncer->sync($page, $per_page, $last_updated_at);
				$page++;
			}
		}

		update_option( 'mf_sync_bookkeeping_last_updated', $new_last_update_at);

        $lock->clear_lock();

		MfLog::log('notice', 'Cron Ended - Bookkeeping Account Sync');
	}

	function do_mf_sync_orders_latest() {
        $lock = new Moreflo_Lock( __FUNCTION__ );
        if ( ! $lock->lock() ) {
            MfLog::log( 'warning', 'Cron lock in place, skipping cron' );
            return;
        }

		MfLog::log(DEBUG, 'Cron Started - Orders Sync Latest');

		$this->set_hook_save_order( false );

		$page_size = 10;
		$mapper = new MfMapper();
		$now = mf_get_current_time();
		$last_updated_at = get_option( 'mf_orders_sync_last_updated' );
		if ( ! $last_updated_at ) {
			$last_updated_at = $now;
		}

		$status = $mapper->pull_orders($last_updated_at, 0, $page_size);

		if(!empty($status['total']) && $status['total'] > $page_size) {
			$total_pages = $status['total'] / $page_size;

			for ($page = 1; $page < $total_pages; $page++) {
				$mapper->pull_orders($last_updated_at, $page, $page_size);
			}
		}

		update_option(  'mf_orders_sync_last_updated' , $now );

        $lock->clear_lock();

		MfLog::log(DEBUG, 'Cron Ended - Orders Sync Latest');
	}

	function do_mf_sync_orders() {
		MfLog::log(DEBUG, 'Cron Started - Orders Sync');

		$orders = Order::get_all();

		if(!empty($orders) && is_array($orders)) {

			foreach ($orders as $order) {

				if($order instanceof WC_Order) {
					
					$mf_order_data = get_post_meta($order->get_order_number(),'mf_order', true);
					$invalidOrders = get_post_meta($order->get_order_number(), 'multiple_bookable_orders', true);

					if(!empty($mf_order_data[MF_ORDER_ID]) && !empty($mf_order_data[MF_ORDER_TYPE]) && empty($invalidOrders)) {
						if($mf_order_data[MF_ORDER_TYPE] == SERVICE) {
							$order_response = $this->api->get_service_order($mf_order_data[MF_ORDER_ID]);
						}
						if($mf_order_data[MF_ORDER_TYPE] == NORMAL) {
							continue; // These orders are synced with latest
							$order_response = $this->api->get_order($mf_order_data[MF_ORDER_ID]);
						}

						$mf_order = Order::transform_mf_order($order_response);

						if(!empty($order_response) && $mf_order && $mf_order->State && $mf_order->PaymentStatus) {
							$order_instance = new Order($order);
							$order_instance->update_order($mf_order->State, $mf_order->PaymentStatus, $mf_order_data[MF_ORDER_TYPE]);

							if($mf_order_data[MF_ORDER_TYPE] == SERVICE) {
								$bookableOrders['StartTime'] = $mf_order->BookingStartTime;
								$bookableOrders['StopTime'] = $mf_order->BookingEndTime;
								$result;

								$mapper = new MfMapper();
								$mf_order_infor = $mapper->transforms_order_data($bookableOrders, 'StartTime', 'StopTime');
								
								$old_hash = get_post_meta($mf_order_data[MF_ORDER_ID], MF_ORDER_INFOR_HASH, true);
								$new_hash = generate_order_hash($mf_order_infor);

								if($old_hash != $new_hash) {
									$order_instance = new Order($order);	
									$result = $order_instance->UpdateTimeSlots($mf_order_infor);
								}

								if(isset($result)) {
									update_post_meta($mf_order_data[MF_ORDER_ID], MF_ORDER_INFOR_HASH, $new_hash);
								}
							}
						}
					}
				}
			}
		}

		MfLog::log(DEBUG, 'Cron Ended - Orders Sync');
	}

	function enqueue_admin_scripts() {
		$mf_settings = get_option(MOREFLO_API_URL);

		if(!empty($mf_settings[MF_SYNC_ITEMS_PER_PAGE])) {
			$items_per_page = $mf_settings[MF_SYNC_ITEMS_PER_PAGE];
		} else {
			$items_per_page = 20;
		}

		wp_register_script( MF_SYNC, plugins_url( 'assets/js/mf_sync.js', __FILE__ ), array( 'jquery' ), '1.0', true );
		wp_enqueue_script( MF_SYNC );
		wp_localize_script( MF_SYNC, 'mf_sync_script_vars', array(
				'sku_required' => __( 'SKU must be set in the Inventory tab.', 'wpmf' ),
				'account_code_required' => __( 'Account code must be selected in the MoreFlo tab.', 'wpmf' ),
				'title_required' => __( 'Product name must be set.', 'wpmf' ),
				'regular_price_required' => __( 'Regular price must be set in the General tab.', 'wpmf' ),
				'variants_sku_price_required' => __( 'SKU and Regular price must be set in all the variants.', 'wpmf' ),
				'sync_starting_to_process_page' => __( 'Starting to process page', 'wpmf' ),
				'sync_completed_processing_page' => __( 'Completed the processing page', 'wpmf' ),
				'sync_completed' => __( 'Sync completed with MoreFlo.', 'wpmf' ),
				'ajax_url' => admin_url( 'admin-ajax.php' ),
				'mf_sync_items_per_page' => $items_per_page
			)
		);
	}

    public function create_plugin_admin()
    {
        $page_title = __('WP MoreFlo Sync', 'wpmf');
        $menu_title = __('MoreFlo Sync', 'wpmf');
        $capability = 'manage_options';
        $slug = 'wp_moreflo_admin';
        $callback = array($this, 'plugin_settings_admin_content');
        $icon = plugin_dir_url(__FILE__) . 'assets/img/wp_moreflo_logo.png';
        $position = 100;

	    if ( 'inactive' === Moreflo_Admin_Status::get_sync_cron_status() ) {
			$menu_title .= ' <span class="awaiting-mod">!</span>';
	    }


        add_menu_page($page_title, $menu_title, $capability, $slug, $callback, $icon, $position);
    }

    public function plugin_settings_admin_content()
    {
	    require_once( plugin_dir_path( __FILE__ ) . 'views/mf-welcome.php' );
	    // $this->do_mf_sync_orders();
    }

    public function mf_sync_cron_schedule($schedules)
    {
        $syncing_interval = get_option(MOREFLO_API_URL);
        $syncing_sec = $syncing_interval['pre_syncing_int'];

        $schedules['every_one_hr'] = array(
            INTERVAL => $syncing_sec,
            DISPLAY => __('Every' . $syncing_sec . 'Minutes', 'wpmf'),
        );
        return $schedules;
    }

    function is_full_product($article) {
	    if(
	            isset($article[ARTICLENUMBER]) && !empty($article[ARTICLENUMBER]) &&
	            // isset($article[ARTICLENAME]) && !empty($article[ARTICLENAME]) &&
	            isset($article[ACCOUNTCODEID]) && !empty($article[ACCOUNTCODEID]) &&
	            // isset($article[RETAILPRICE]) && !is_null($article[RETAILPRICE]) && $article[RETAILPRICE] != '' && $article[RETAILPRICE] > -1 &&
	            isset($article[ARTICLETYPE]) && !empty($article[ARTICLETYPE])
        ) {
	        return true;
        }

	    return false;
    }

	function save_product($product_id, $product)
	{
		if(get_post_meta($product_id, 'tempary_lock', true)){
			MfLog::log(DEBUG, 'Save product skipping, becuase due to tempory lock' . $product_id);
			return;
		}
        $article_instance = new MfArticles();
		$articles = array();
		$ignoreStatus = array(ARTICLEDRAFT, 'auto-draft');
		$productstatus = $product->get_status();

		if($product instanceof WC_Product_Variable && !in_array($productstatus, $ignoreStatus)) {

			$variations = $product->get_children();

			if(is_array($variations) && !empty($variations)) {

				foreach ($variations as $variant_id) {
					$articles[$variant_id] = $article_instance->variantToArticle($product, $variant_id);
				}

			}
		} else {
			$articles[$product_id] = $article_instance->productToArticle($product);
		}

		if(!empty($articles)) {

			foreach ($articles as $id => $article) {
				MfLog::log(DEBUG, 'Saving article');
				MfLog::log(DEBUG, json_encode($article));

				if(!empty($article) && $this->is_full_product($article)) {

					$old_hash = get_post_meta($id, MF_HASH, true);
					$new_hash = generate_article_hash($article);
					if($old_hash != $new_hash) {
						if($old_hash) {
							$response =  $this->api->put_articles($article);
						} else {
							$response =  $this->api->post_articles($article);
						}


						if($response) {
							update_post_meta($id, 'mf_hash', $new_hash);
							update_post_meta($id, 'mf_sync_status', __('Synced', 'wpmf'));
							update_post_meta($id, 'mf_last_sync_response', $response);
							MfLog::log(DEBUG, 'Product pushed and meta data updated');

							Moreflo_Admin_Status::set_sync_status_from_response( $id, $response );
						}
					} else {
						MfLog::log(DEBUG, 'Hash verification failed');
					}
				} else {
					MfLog::log(DEBUG, 'Article updated failed due empty article or is not full article');
				}
			}
		}

	}

	public function manage_shop_order_posts_columns( $columns ) {
		return array_merge($columns, ['apps_order_id' => 'Apps']);
	}

	public function manage_shop_order_posts_custom_column( $column_key, $post_id ) {
		if ( 'apps_order_id' == $column_key ) {
			$mf_apps_order_id = mf_get_apps_order_id( $post_id );
			if ( ! is_null( $mf_apps_order_id ) ) {
				echo '#'. $mf_apps_order_id;
			}
		}
	}
}

$mfadmin = new MfAdmin();
