<?php

use \Moreflo\Order;

class MfMapper
{
	private $settings;
	private $shipping_product_id;

	public function __construct() {
		$this->settings = get_option( 'moreflo_api_url' );
		$this->shipping_product_id = ( isset( $this->settings['mf_shipping_product_id'] ) )
			? $this->settings['mf_shipping_product_id']
			: '';
	}

	function process_stock_update($response) {

		$article_instance = new MfArticles();
		$articles = json_decode($response, true);

		if(!empty($articles['list']) && is_array($articles['list'])) {

			foreach ( $articles['list'] as $article ) {
				if(!empty($article[ARTICLENUMBER]) && $article['Success']) {
					$product_id = wc_get_product_id_by_sku($article[ARTICLENUMBER]);
					$article_instance->update_stock($product_id, $article[QUANTITY]);
					return true;
				}else{
					return $article['ExtraInformation'];
				}

			}

		}
	}

	function pull_articles($page, $pageSize = 100, $params = []) {

		$api = new MfApi();
		$status = array('add' => 0, 'new' => 0, 'update' => 0, 'page' => 0, 'total' => 0);

		$params = array_merge($params, array(
			'page' => $page,
			'pageSize' =>  $pageSize
		));

		$articles = $api->get_articles_new($params);

		if(!empty($articles) && !empty($articles['Total'])) {
			$status['page'] = $articles['Page'];
			$status['total'] = $articles['Total'];
		}

		if(!empty($articles['list']) && is_array($articles['list'])) {

			foreach ( $articles['list'] as $article ) {

				if(!empty($article['Article'])) {
					$action = $this->process_pulled_article($article['Article']);

					if($action !== false) {
						$status[$action]++;
					}
				}

			}

		}

		return $status;

	}

	public function process_pulled_article($article) {

		$return = false;

		$article = $this->transform_article_to_product($article);

		$sku = $article['Sku'];
		$mf_is_web_shop = $article[ISWEBSHOPARTICLE];
		$mf_is_active = $article[ISACTIVEARTICLE];

		MfLog::log(DEBUG, 'Processing article no: ' . $sku);

		if ( $this->shipping_product_id == $sku ) {
			return false;
		}

		$product_id = wc_get_product_id_by_sku( $sku );

		$api_options = get_option(MOREFLO_API_URL);

		if (
			! empty( $api_options['sync_only_product_stock_amount'] ) &&
			$api_options['sync_only_product_stock_amount'] == 'enabled'
		) {
			if ( ! empty( $product_id ) ) {
				$article_instance = new MfArticles();
				$article_instance->updateOnlyStock( $product_id, $article );
				$this->update_product_sync_status( $product_id );
				return 'update';
			} else {
				MfLog::log(DEBUG, 'Skipping due to newArticle/onlyStock.');
				return false;
			}
		}

		if ( !$mf_is_web_shop || !in_array($article[ARTICLETYPE], array(TYPEGOODS, TYPESERVICE)) ) {
			MfLog::log(DEBUG, 'Skipping due to !webshop/shipping/!ArticleType.');
			MfLog::log(DEBUG, print_r($article, true));

			$this->maybe_set_as_draft( $product_id );

			return false;
		}

		$article_instance = new MfArticles();

		if(empty($product_id)) {
			if (
				$mf_is_active != ARTICLEPUBLISH
			) {
				MfLog::log(DEBUG, 'Skipping due to !active');
				MfLog::log(DEBUG, print_r($article, true));

				$this->maybe_set_as_draft( $product_id );

				return false;
			}
			$product_id = $article_instance->add($article);
			$return = 'add';
			MfLog::log(DEBUG, 'Product created in WC: ' . $sku);
		} else {
			/**
			 * Todo if customer requested update product to 
			 * draft what ever article are pubshlied to draft based on !active
			 */
			update_post_meta($product_id, 'tempary_lock', 'yes');
			$article_instance->update($product_id, $article);
			$return = 'update';
			MfLog::log(DEBUG, json_encode($article));
			MfLog::log(DEBUG, 'Product updated in WC: ' . $sku);
			delete_post_meta($product_id, 'tempary_lock', 'yes');
		}

		$this->update_product_sync_status( $product_id );

		MfLog::log(DEBUG, 'Processed article no: ' . $sku);

		return $return;
	}

	private function update_product_sync_status( $product_id ) {
		$product = wc_get_product($product_id);

		if($product instanceof WC_Product) {
			$article_instance = new MfArticles();
			$article = $article_instance->productToArticle($product);
			$new_hash = generate_article_hash($article);
			update_post_meta($product_id, MF_HASH, $new_hash);
			update_post_meta($product_id, MF_SYNC_STATUS, __('Synced', 'wpmf'));

			Moreflo_Admin_Status::set_sync_status_ok( $product_id );
		}
	}

	function pull_orders( $last_updated_at, $page, $page_size ) {
		$api = new MfApi();
		$status = array('update' => 0, 'ignore' => 0, 'page' => 0, 'total' => 0);

		$orders = $api->get_orders( $last_updated_at, $page, $page_size );

		if(!empty($orders) && !empty($orders['Total'])) {
			$status['page'] = $orders['Page'];
			$status['total'] = $orders['Total'];
		}

		if(!empty($orders['list']) && is_array($orders['list'])) {
			foreach ( $orders['list'] as $order ) {
				if(!empty($order['Order'])) {
					$action = $this->process_pulled_order($order['Order']);

					if($action !== false) {
						$status[$action]++;
					}
				}
			}
		}

		return $status;
	}

	public function process_pulled_order( $mf_order ) {
		$r = 'ignore';
		$mf_id = $mf_order['Id'];

		MfLog::log(DEBUG, 'Processing order ' . $mf_id);

		$wc_orders = wc_get_orders([
			'limit' => 1,
			'meta_key' => 'mf_id',
			'meta_value' => $mf_id,
		]);

		if ( ! empty( $wc_orders ) && $mf_order && $mf_order['State'] && $mf_order['PaymentStatus'] ) {
			$wc_order = array_shift( $wc_orders );
			MfLog::log(DEBUG, 'Found WC order '. $wc_order->get_id());

			$order_instance = new Order($wc_order);
			$order_instance->update_order($mf_order['State'], $mf_order['PaymentStatus'], 'normal');
			$r = 'update';
		}

		MfLog::log(DEBUG, 'Processed order ' . $mf_id .' ('. $r .')');

		return $r;
	}

	private function maybe_set_as_draft( $product_id ) {
		if ( ! isset( $this->settings['sync_publish_products_status'] ) ) {
			return;
		}
		if ( 'enabled' != $this->settings['sync_publish_products_status'] ) {
			return;
		}

		if ( ! empty( $product_id ) ) {
			$product = wc_get_product( $product_id );
			if ( 'publish' == $product->get_status( 'edit' ) ) {
				if ( 'simple' == $product->get_type() ) {
					$product->set_status( 'draft' );
					$product->save();
					MfLog::log(DEBUG, 'Product set to draft (id: '. $product_id .')');
				} elseif ( 'variation' == $product->get_type() ) {
					$product->set_status( 'private' );
					$product->save();
					MfLog::log(DEBUG, 'Product variation set to private (id: '. $product_id .')');
				}
			}
		}
	}


	function transform_article_to_product($article_item)
	{
		$article = array();

		if (is_array($article_item) && !empty($article_item)) {

			$article_status = $article_item[ISACTIVE];

			if ($article_status) {
				$article[ISACTIVEARTICLE] = ARTICLEPUBLISH;
			} else {
				$article[ISACTIVEARTICLE] = ARTICLEDRAFT;
			}

			$article['Sku'] = $article_item[ARTICLENUMBER];
			$article[QUANTITY] = $article_item[QUANTITY];
			$article['Name'] = $article_item[ARTICLENAME];
			$article[ISSTOCKITEM] = $article_item[ISSTOCKITEM];

			$stk = $article_item[ISSTOCKITEM];

			if ($stk) {
				$article['manage_stock'] = "yes";
			} else {
				$article['manage_stock'] = "no";
			}

			$article['ArticleTextHtml'] = is_null( $article_item['ArticleTextHtml'] ) ? '' : $article_item['ArticleTextHtml'];
			$article['Description'] = $article_item['ArticleDescription'];
			$article['GroupId'] = $article_item['GroupId'];
			$article[RETAILPRICE] = $article_item[RETAILPRICE];
			$article[RECOMMENDEDPRICE] = $article_item[RECOMMENDEDPRICE];
			$article[QUANTITY] = $article_item[QUANTITY];
			$article[ISWEBSHOPARTICLE] = $article_item[ISWEBSHOPARTICLE];
			$article[ARTICLETYPE] = $article_item[ARTICLETYPE];
			$article[ACCOUNTCODEID] = $article_item[ACCOUNTCODEID];
			$article[BARCODE] = $article_item[BARCODE];//Adding barcode 19-02-2019
			$article['Files'] = $article_item['Files'];//Adding barcode 19-02-2019

			
			$article['BarcodeAlternatives'] = isset( $article_item['BarcodeAlternatives'] ) ? $article_item['BarcodeAlternatives'] : [];
			
			if(is_numeric($article_item[ARTICLETYPE]) && $article_item[ARTICLETYPE] > -1) {
				switch($article_item[ARTICLETYPE]) {
					case 0:
						$article[ARTICLETYPE] = TYPEGOODS;
						break;
					case 1:
						$article[ARTICLETYPE] = TYPESERVICE;
						if(isset($article_item['IsBookable']) && $article_item['IsBookable'] == true){
							$article['IsBookable'] = $article_item['IsBookable'];
							foreach($article_item['ArticleBookingSettings'] as $key => $value){
								$bookingTimeFields = ["DefaultDuration","PaddingTime","LeadTime","IntervalTime","SchedulingWindow"];
								if(in_array($key, $bookingTimeFields)){
									$convertTimeObj = $this->timeConvertion($value, $key);
									$article[$key] = $convertTimeObj['time'];
									$article[$key.'Unit'] = $convertTimeObj['unit'];
								}else{
									$article[$key] = $value;
								}
							}
						}
						break;
				}
			}

			$article = apply_filters( 'moreflo_sync_transform_article_to_product', $article, $article_item );
		}

		return $article;
	}
	
	function timeConvertion($time, $type){
		/**
		 * Below shown are unit we can set for relevant to each type which is define appointment plugin.
		 * 
		 * DefaultDuration - min, hours, days, month
		 * PaddingTime - min, hours, month
		 * IntervalTime - min, hours
		 * LeadTime - hours, days, month
		 * SchedulingWindow - hours, days, month
		 * 
		 */
		$share_unit_type_1 = ['IntervalTime', 'PaddingTime'];
		$share_unit_type_2 = ['LeadTime', 'SchedulingWindow'];

		if($time % (30*24*60*60) == 0 && $type !='IntervalTime')
		{
			return [
				'time' => round($time/(30*24*60*60)),
				'unit'=> 'month'
				];
		}
		elseif($time % (24*60*60) == 0 && !in_array($type, $share_unit_type_1))
		{
			return [
				'time' => round($time/(24*60*60)),
				'unit'=> 'day'
				];
		}
		elseif ($time % (60*60) == 0 || in_array($type, $share_unit_type_2))
		{
			return [
				'time' => round($time/(60*60)),
				'unit'=> 'hour'
			];
		}
		elseif ($time % (60) == 0 || $time >= 60 && !in_array($type, $share_unit_type_2))
		{
			return [
				'time' => round($time/(60)),
				'unit'=> 'minute'
			];
		}
		elseif ($time < 60 && !in_array($type, $share_unit_type_2))
		{
			return [
				'time' => 1,
				'unit'=> 'minute'
			];
		}
	}

	public function transforms_order_data($bookingOrder, $key1, $key2){

        $bookingData = array(
			'StartDate'			=>  date_format(date_create($bookingOrder[$key1]), 'Y-m-d' ),
			'StopDate'			=>  date_format(date_create($bookingOrder[$key2]), 'Y-m-d' ),
			'StartTime'			=>  date_format(date_create($bookingOrder[$key1]), 'H:i' ),
			'StopTime'			=>  date_format(date_create($bookingOrder[$key2]), 'H:i' ),
		);
		
		return $bookingData;

	}
}
