<?php


class MfSettings {

	public function __construct() {
		add_action( 'admin_menu', array( &$this, 'add_sub_menu' ) );
	}


	public function add_sub_menu() {

		$parent_slug = 'wp_moreflo_admin';
		$page_title = 'WP Moreflo';
		$menu_title = 'Settings';
		$capability = 'manage_options';
		$slug = 'wp_moreflo_settings';
		$callback = array( $this, 'plugin_field_setup_content' );

		add_submenu_page( $parent_slug, $page_title, $menu_title, $capability, $slug, $callback, 1 );
	}

	public function plugin_field_setup_content() {

		require_once( plugin_dir_path( __FILE__ ) . 'views/mf-setup-view.php' );

		if ( isset( $_POST['add_api_url'] ) ) {

			$mf_options_api = get_option( MOREFLO_API_URL, [] );
			$mf_options_api['log_enabled'] = isset( $_POST['mf_error_log'] )
				? sanitize_text_field( $_POST['mf_error_log'] )
				: '';
			$mf_options_api[MF_SYNC_ARTICLES_INTERVAL] = sanitize_text_field( $_POST[MF_SYNC_ARTICLES_INTERVAL] );
			$mf_options_api['mf_sync_items_per_page'] = sanitize_text_field( $_POST['mf_sync_items_per_page'] );
			$mf_options_api[MF_SYNC_STATUS] = sanitize_text_field( $_POST[MF_SYNC_STATUS] );
			$mf_options_api['sync_publish_products_status'] = 'enabled' == $_POST['sync_publish_products_status']
				? 'enabled'
				: 'disabled';
			$mf_options_api['sync_only_product_stock_amount'] = 'enabled' == $_POST['sync_only_product_stock_amount']
				? 'enabled'
				: 'disabled';
			$mf_options_api[ISPRICEACTIVE] = sanitize_text_field( $_POST['sync_price_select'] );
			$mf_options_api['mf_shipping_product_id'] = sanitize_text_field( $_POST['mf_shipping_product_id'] );
			$mf_options_api['mf_part_refund_product_id'] = sanitize_text_field( $_POST['mf_part_refund_product_id'] );
			$mf_options_api['api_test_url'] = sanitize_text_field( $_POST['api_test_url'] );
			$mf_options_api['api_test_user'] = sanitize_text_field( $_POST['api_test_user'] );
			$mf_options_api['api_test_pw'] = sanitize_text_field( $_POST['api_test_pw'] );
			$mf_options_api['api_production_url'] = sanitize_text_field( $_POST['api_production_url'] );
			$mf_options_api['api_production_user'] = sanitize_text_field( $_POST['api_production_user'] );
			$mf_options_api['api_production_pw'] = sanitize_text_field( $_POST['api_production_pw'] );
			$mf_options_api['is_active'] = sanitize_text_field( $_POST['url_select'] );
			$mf_options_api['sync_article_description'] = 'enabled' == $_POST['sync_article_description']
				? 'enabled'
				: 'disabled';
			$mf_options_api['is_pic_active'] = sanitize_text_field( $_POST['sync_pic_select'] );
			$mf_options_api['is_product_title_active'] = sanitize_text_field( $_POST['sync_product_title_select'] );
			$mf_options_api['is_tax_class_active'] = sanitize_text_field( $_POST['sync_product_tax_class'] );
			$mf_options_api['is_product_categories_active'] = sanitize_text_field( $_POST['sync_product_categories'] );
			$mf_options_api['is_campaigns_active'] = sanitize_text_field( $_POST['sync_campaigns'] );
			$mf_options_api['is_receipt_active'] = sanitize_text_field( $_POST['sync_receipt_active'] );
			$mf_options_api['sync_new_orders_to_apps'] = 'disabled' == $_POST['sync_new_orders_to_apps']
				? 'disabled'
				: 'enabled';
			$mf_options_api['mf_company_corp_no'] = sanitize_text_field( $_POST['mf_company_corp_no'] );
			$mf_options_api['mf_company_name'] = sanitize_text_field( $_POST['mf_company_name'] );
			$mf_options_api['mf_bookkeeping_account_shipping'] = sanitize_text_field( $_POST['mf_bookkeeping_account_shipping'] );
			// $mf_options_api['mf_bookkeeping_account_tax'] = sanitize_text_field( $_POST['mf_bookkeeping_account_tax'] );

			// Bookkeeping accounts
			$mf_options_bookkeeping_account_payments = [];
			foreach ( $_POST as $key => $value ) {
				$matches = [];
				if ( ! preg_match( '/^mf_bookkeeping_account_payments_(.+)$/', $key, $matches ) ) {
					continue;
				}

				$mf_options_bookkeeping_account_payments[ $matches[1] ] = sanitize_text_field( $value );
			}

			$mf_options_api['bookkeeping_account_payments'] = $mf_options_bookkeeping_account_payments;

			update_option( MOREFLO_API_URL, $mf_options_api );

			$mf_api = new MfApi();
			$response = $mf_api->get_account_codes();

			if($response) {

				$data = json_decode($response);

				if($data && isset($data->list) && is_array($data->list)) {
					delete_option('mf_account_ids');
					update_option('mf_account_ids', $data->list);
					$this->sync_tax_rules($data->list);
				}

				$this->setup_cron_jobs($_POST[MF_SYNC_STATUS], $mf_options_api[MF_SYNC_ARTICLES_INTERVAL]);
			}

			update_option( MF_SYNC_CRON_CHECK_EMAIL, false );

			wp_redirect( home_url( '/wp-admin/admin.php?page=wp_moreflo_settings/' ) );

			exit;
		}
	}

	private function setup_cron_jobs($status, $recurrence) {

		//Removing the old cron job if it exists
		if (wp_next_scheduled( 'mf_sync_cron_schedule' ) ) {
			wp_clear_scheduled_hook( 'mf_sync_cron_schedule' );
		}

		wp_clear_scheduled_hook('mf_sync_articles');
		wp_clear_scheduled_hook('mf_sync_orders');

		if($status == 'enabled') {
			$cron = wp_schedule_event( time(), $recurrence, 'mf_sync_articles' );
			$orders_cron = wp_schedule_event( time(), $recurrence, 'mf_sync_orders' );

			if($cron) {
				MfLog::log(DEBUG, 'mf_sync_articles Cron job added');
			} else {
				MfLog::log(DEBUG, 'mf_sync_articles Cron job adding failed');
			}

			if($orders_cron) {
				MfLog::log(DEBUG, 'mf_sync_orders Cron job added');
			} else {
				MfLog::log(DEBUG, 'mf_sync_orders Cron job adding failed');
			}

		} else {
			MfLog::log(DEBUG, 'mf_sync_articles Cron job disabled');
			MfLog::log(DEBUG, 'mf_sync_orders Cron job disabled');
		}

	}

	
	public function sync_tax_rules($dataResponse) {
		
		global $wpdb;
		$wc_tax_rate_cls = [];
		$wc_tax_rates = [];
		$mf_wc_tax_rules = [];
		$pre_api_options = get_option(MOREFLO_API_URL);
		
		if(isset($pre_api_options['is_tax_class_active']) && $pre_api_options['is_tax_class_active'] == 'enabled'){
			$option_tax_tables = [
				'mf_tax_rates_cls' => ['tax_rate_class_id','wc_tax_rate_classes'],
				'mf_tax_rates_ids' => [ 'tax_rate_id','woocommerce_tax_rates'] 
			];
			foreach ($option_tax_tables as $tblName => $tblDetail) {
				$mf_option_result = get_option($tblName);
				if(!empty($mf_option_result)){
					foreach ($mf_option_result as $Id) {
						$deleted = $wpdb->delete( "{$wpdb->prefix}$tblDetail[1]" , array( $tblDetail[0] => $Id ), array( '%d' ) );
					}
				}
			}

			foreach ($dataResponse as $listitem) {
				$tax_class_name = $listitem->AccountCode->Name;
				$tax_class_rate = $listitem->AccountCode->VatRate;
				$tax_class_rate_name = $listitem->AccountCode->Description;
				$tax_class_rate_id = $listitem->AccountCode->Id;
				$tax_class_slug = sanitize_title(str_replace(" ", "-", strtolower($tax_class_name)));
				$tbl_wc_tax_rate_cls = $wpdb->prefix.'wc_tax_rate_classes';
				$wc_tax_rate_cls_data = array(
					'name' => 'MF '.$tax_class_name, 
					'slug' => 'mf-'.$tax_class_slug
				);
				$wc_tax_rate_cls_format = array('%s','%s');

				$tbl_wc_tax_rates = $wpdb->prefix.'woocommerce_tax_rates';
				$wc_tax_rates_data = array(
					'tax_rate_country' => '', 
					'tax_rate_state' => '',
					'tax_rate' => $tax_class_rate.'.0000',
					'tax_rate_name' => $tax_class_rate_name,
					'tax_rate_priority' => 1,
					'tax_rate_compound' => 0,
					'tax_rate_shipping' => 1,
					'tax_rate_order' => 0,
					'tax_rate_class' => 'mf-'.$tax_class_slug						
				);
				$wc_tax_rates_format = array('%s','%s','%s','%s','%d','%d','%d','%d','%s');
				
				$insert_wc_tax_rate_classes_id = mf_insert_row( $tbl_wc_tax_rate_cls, $wc_tax_rate_cls_data, $wc_tax_rate_cls_format );
				if($insert_wc_tax_rate_classes_id){
					array_push($wc_tax_rate_cls, $insert_wc_tax_rate_classes_id);
				}
				$insert_wc_tax_rate_id = mf_insert_row( $tbl_wc_tax_rates, $wc_tax_rates_data, $wc_tax_rates_format );
				if($insert_wc_tax_rate_id){
					$mf_wc_tax_rules[$tax_class_rate_id] = ['mf-'.$tax_class_slug, $tax_class_rate];
					array_push($wc_tax_rates, $insert_wc_tax_rate_id);
				}
			}
			if(!empty($wc_tax_rate_cls)){
				update_option('mf_tax_rates_cls', $wc_tax_rate_cls);
			}
			if(!empty($wc_tax_rates)){
				update_option('mf_tax_rates_ids', $wc_tax_rates);
			}
			if(!empty($mf_wc_tax_rules)){
				update_option('mf_wc_tax_rules', $mf_wc_tax_rules);
			}
		}
	}
}

$mfsetting = new MfSettings();

