<?php
class MfUpdate
{
	public function __construct()
	{
		add_filter('plugins_api', array(
			&$this,
			'mf_plugin_info'
		), 20, 3);
		add_filter('site_transient_update_plugins', array(
			&$this,
			'mf_push_updates'
		));
		add_filter('upgrader_process_complete', array(
			&$this,
			'mf_after_update'
		), 10, 2);
	}
	function mf_plugin_info($res, $action, $args)
	{
		if (($action !== 'plugin_information') || (PLUGIN_SLUG !== $args->slug)) {
			return false;
		}
		if (!($remote = get_transient(PLUGIN_UPGRADE_TRANS))) {
			$remote = wp_remote_get('https://downloadsflo.northmill.com/plugin/info.json', array(
				'timeout' => 10,
				'headers' => array(
					'Accept' => 'application/json'
				)
			));
			if (!is_wp_error($remote) && isset($remote[RESPONSE]['code']) && $remote[RESPONSE]['code'] == 200 && !empty($remote['body'])) {
				set_transient(PLUGIN_UPGRADE_TRANS, $remote, 43200); // 12 hours cache
			}
		}
		if ($remote) {
			$remote              = json_decode($remote['body']);
			$res                 = new stdClass();
			$res->name           = $remote->name;
			$res->slug           = PLUGIN_SLUG;
			$res->version        = $remote->version;
			$res->tested         = $remote->tested;
			$res->requires       = $remote->requires;
			$res->author         = '<a href="https://www.moreflo.com">Moreflo</a>';
			$res->author_profile = 'https://www.moreflo.com';
			$res->download_link  = $remote->download_url;
			$res->trunk          = $remote->download_url;
			$res->last_updated   = $remote->last_updated;
			$res->sections       = array(
				'changelog' => $remote->sections->changelog
			);
			if (!empty($remote->sections->screenshots)) {
				$res->sections['screenshots'] = $remote->sections->screenshots;
			}
			$res->banners = array(
				'low' => 'https://downloadsflo.northmill.com/wp-content/uploads/2017/04/moreflo-system-1024x532.png',
				'high' => 'https://downloadsflo.northmill.com/wp-content/uploads/2017/04/moreflo-system-1024x532.png'
			);
			return $res;
		}
		return false;
	}
	function mf_push_updates($transient)
	{
		if (empty($transient->checked)) {
			return $transient;
		}

		if (!($remote = get_transient(PLUGIN_UPGRADE_TRANS))) {
			$remote = wp_remote_get('https://downloadsflo.northmill.com/plugin/info.json', array(
				'timeout' => 10,
				'headers' => array(
					'Accept' => 'application/json'
				)
			));
			if (!is_wp_error($remote) && isset($remote[RESPONSE]['code']) && $remote[RESPONSE]['code'] == 200 && !empty($remote['body'])) {
				set_transient(PLUGIN_UPGRADE_TRANS, $remote, 120); // 12 hours cache
			}
		}

		$plugin_data    = get_plugin_data(plugin_dir_path(__FILE__) . '/wp_moreflo.php');
		$plugin_version = $plugin_data['Version'];

		if ($remote && !is_wp_error($remote) && isset($remote['body'])) {
			$remote = json_decode($remote['body']);
			if ($remote && version_compare($plugin_version, $remote->version, '<') && version_compare($remote->requires, get_bloginfo('version'), '<')) {
				$res                               = new stdClass();
				$res->slug                         = PLUGIN_SLUG;
				$res->plugin                       = plugin_basename(__DIR__) . '/wp_moreflo.php';
				$res->new_version                  = $remote->version;
				$res->tested                       = $remote->tested;
				$res->package                      = $remote->download_url;
				$res->compatibility                = new stdClass();
				$transient->response[$res->plugin] = $res;
			}
		}

		return $transient;
	}
	function mf_after_update($upgrader_object, $options)
	{
		if ($options['action'] == 'update' && $options['type'] === 'plugin') {
			delete_transient(PLUGIN_UPGRADE_TRANS);
		}
	}
}
$mfupdate = new MfUpdate();
