<?php
define('ISOPENARTICLE', 'IsOpenArticle');
if(!function_exists('mf_is_woocommerce_bookings_active')) {
	function mf_is_wc_bookings_active() {
		return is_plugin_active('woocommerce-bookings/woocommerce-bookings.php');
	}
}

if(!function_exists('mf_is_woocommerce_appointments_active')) {
	function mf_is_wc_appointments_active() {
		return is_plugin_active('woocommerce-appointments/woocommerce-appointments.php');
	}
}

if(!function_exists('mf_get_account_codes')) {
	function mf_get_account_codes() {

		$data = get_option('mf_account_ids');
		$return['0'] = __('Please select a account code.', 'wpmf');

		if($data && is_array($data) && !empty($data)) {
			foreach ($data as $account_code) {
				if(!empty($account_code->AccountCode->Id) && !empty($account_code->AccountCode->Name)){
					$return[$account_code->AccountCode->Id] = $account_code->AccountCode->Name;
				}
			}
		}

		return $return;
	}
}

if(!function_exists('generate_article_hash')) {
	function generate_article_hash( $article ) {

		$hash_string = '';
		$complied_string = '';

		if ( is_array( $article ) && ! empty( $article ) ) {

			$hashed_properties = array(
				ARTICLENUMBER,
				ARTICLENAME,
				UNIT,
				ISACTIVE,
				UPDATEONEXISTING,
				//'Quantity',
				ISWEBSHOPARTICLE,
				ISSTOCKITEM,
				ISOPENARTICLE,
				ACCOUNTCODEID,
				RECOMMENDEDPRICE,
				RETAILPRICE,
				ARTICLETYPE
			);

			foreach ( $hashed_properties as $key ) {

				if ( isset( $article[ $key ] ) ) {
					$complied_string .= $article[ $key ];
				}
			}
			$hash_string = wp_hash($complied_string, 'nonce');
		}

		return $hash_string;
	}
}

if(!function_exists('generate_order_hash')) {
	function generate_order_hash( $order ) {

		$hash_string = '';

		if ( is_array( $order ) && ! empty( $order ) ) {
			$hash_string = wp_hash(json_encode($order), 'nonce');
		}

		return $hash_string;
	}
}

if(!function_exists('mf_get_current_time')) {
	function mf_get_current_time() {
		$date = new DateTime( "now", new DateTimeZone( 'Europe/Stockholm' ) );
		return $date->format( 'Y-m-d\TH:i:s' );
	}
}

if(!function_exists('mf_insert_row')) {
	function mf_insert_row( $table, $data, $format ) {
		global $wpdb;
		$result = $wpdb->insert( $table, $data, $format );
		if($result){
			return $wpdb->insert_id;
		}
	}
}

/**
 * Round a number using the built-in `round` function, but unless the value to round is numeric
 * (a number or a string that can be parsed as a number), apply 'floatval' first to it
 * (so it will convert it to 0 in most cases).
 *
 * This is needed because in PHP 7 applying `round` to a non-numeric value returns 0,
 * but in PHP 8 it throws an error. Specifically, in WooCommerce we have a few places where
 * round('') is often executed.
 *
 * @param mixed $val The value to round.
 * @param int   $precision The optional number of decimal digits to round to.
 * @param int   $mode A constant to specify the mode in which rounding occurs.
 *
 * @return float The value rounded to the given precision as a float, or the supplied default value.
 */
function mf_wc_round( $val, int $precision = 0, int $mode = PHP_ROUND_HALF_UP ) : float {
	if ( ! is_numeric( $val ) ) {
		$val = floatval( $val );
	}
	return round( $val, $precision, $mode );
}

/**
 * Gets Apps order number from WooCommerce order if possible
 * 
 * @param int $order_id
 * @return string|null Apps order number, null on error or not found
 */
function mf_get_apps_order_id( $order_id ) {
	$value = null;
	$response = get_post_meta( $order_id, 'mf_last_sync_response', true );
	if ( $response && ! is_null( $response ) ) {
		$resp_obj = json_decode( $response );
		if ( $resp_obj && ! is_null( $resp_obj ) ) {
			if ( ! empty( $resp_obj->list ) && $resp_obj->list[0]->Order ) {
				$value = $resp_obj->list[0]->Order->Number
					? $resp_obj->list[0]->Order->Number
					: null;
			}
		}
	}

	return $value;
}
