<?php

defined( 'ABSPATH' ) || exit;

class MF_Sync_Bookkeeping_Vat_Sync implements MF_Sync_Sync {
    /**
     * Syncs bookkeeping vat accounts to WooCommerce
     */
    public function sync( $page, $pageSize, $updatedSince ) {
		$api = new MfApi();
		$status = array('delete' => 0, 'ignore' => 0, 'error' => 0, 'add' => 0, 'new' => 0, 'update' => 0, 'page' => 0, 'total' => 1);
		$params = array(
			'page' => $page,
			'pageSize' =>  $pageSize
		);

		$campaigns = $api->get_vat_accounts($params);

		if(!empty($campaigns) && !empty($campaigns['Total'])) {
			$status['page'] = $campaigns['Page'];
			$status['total'] = $campaigns['Total'];
		}

		if(!empty($campaigns['list']) && is_array($campaigns['list'])) {
			$status = $this->process_pulled_vat_accounts( $campaigns['list'], $status );
		}

		return $status;
    }

	public function process_pulled_vat_accounts( $list, $status ) {
		if ( empty( $list ) ) {
			return $status;
		}

        foreach ( $list as $item ) {
            if ( empty( $item['VatAccountSetting'] ) ) {
                continue;
            }
            
            $action = $this->process_pulled_vat_account( $item['VatAccountSetting'] );

            if ( $action !== false ) {
                $status[ $action ]++;
            }
        }

		return $status;
	}

	public function process_pulled_vat_account( $vat_account_setting ) {
        $vat_rate = intval( $vat_account_setting['VatRate'] );
        $account = $vat_account_setting['AccountNumber'];
        MF_Sync_Bookkeeping::set_vat_account( $vat_rate, $account );

		return 'update';
	}
}
