<?php

defined( 'ABSPATH' ) || exit;

class MF_Sync_Campaigns_Factory {
    /**
     * @param string $name
     * @param string $sku
     * @param float $price
     * @return MF_Campaigns_Product
     */
    public static function create_product( $name, $sku, $price, $vat_rate ) {
        return new MF_Campaigns_Product( $name, $sku, $price, $vat_rate );
    }

    /**
     * @param mixed $cart_item
     * @param WC_Cart $cart
     * @return MF_Campaigns_Product[]
     */
    public static function create_products_from_cart_item( $cart_item, $cart ) {
        $r = [];

        for ( $i = 0; $i < $cart_item['quantity']; $i++ ) {
            $name = $cart_item['data']->get_name();
            $sku = $cart_item['data']->get_sku();
            $price = wc_get_price_including_tax( $cart_item['data'] );
            $vat_rate = MF_Sync_Receipts_Helper::get_cart_product_tax_rate( $cart_item['data'], $cart );

            $r[] = self::create_product( $name, $sku, $price, $vat_rate );
        }

        return $r;
    }

    /**
     * @param WP_Coupon $coupon
     * @return MF_Campaigns_Discounter|null Discounter if valid Apps Campaign, else null.
     */
    public static function create_discounter( $coupon ) {
        $discount_type = get_post_meta( $coupon->get_id(), 'discount_type', true );
        $threshold_total = get_post_meta( $coupon->get_id(), 'mf_threshold_total', true );
        $priority = get_post_meta( $coupon->get_id(), 'mf_priority', true );

        if ( 'mf_type_0' == $discount_type ) {
			$skus = json_decode( get_post_meta( $coupon->get_id(), 'mf_parts_sku', true ) );
			$value_x = get_post_meta( $coupon->get_id(), 'mf_value_x', true );
			$value_y = get_post_meta( $coupon->get_id(), 'mf_value_y', true );

            return new MF_Campaigns_Type_0( $coupon, $skus, $value_x, $value_y, $priority, $threshold_total );
        } elseif ( 'mf_type_1' == $discount_type ) {
			$skus = json_decode( get_post_meta( $coupon->get_id(), 'mf_parts_sku', true ) );
			$value_x = get_post_meta( $coupon->get_id(), 'mf_value_x', true );
			$value_y = get_post_meta( $coupon->get_id(), 'mf_value_y', true );

            return new MF_Campaigns_Type_1( $coupon, $skus, $value_x, $value_y, $priority, $threshold_total );
        } elseif ( 'mf_type_2' == $discount_type ) {
			$skus = json_decode( get_post_meta( $coupon->get_id(), 'mf_parts_sku', true ) );
			$value_x = get_post_meta( $coupon->get_id(), 'mf_value_x', true );
			$value_y = get_post_meta( $coupon->get_id(), 'mf_value_y', true );

            return new MF_Campaigns_Type_2( $coupon, $skus, $value_x, $value_y, $priority, $threshold_total );
        } elseif ( 'mf_type_3' == $discount_type ) {
			$skus = json_decode( get_post_meta( $coupon->get_id(), 'mf_parts_sku', true ) );
			$values = json_decode( get_post_meta( $coupon->get_id(), 'mf_parts_value', true ) );

            return new MF_Campaigns_Type_3( $coupon, $skus, $values, $priority, $threshold_total );
        } elseif ( 'mf_type_4' == $discount_type ) {
			$skus = json_decode( get_post_meta( $coupon->get_id(), 'mf_parts_sku', true ) );
			$values = json_decode( get_post_meta( $coupon->get_id(), 'mf_parts_value', true ) );

            return new MF_Campaigns_Type_4( $coupon, $skus, $values, $priority, $threshold_total );
        } elseif ( 'mf_type_5' == $discount_type ) {
			$skus = json_decode( get_post_meta( $coupon->get_id(), 'mf_parts_sku', true ) );
			$values = json_decode( get_post_meta( $coupon->get_id(), 'mf_parts_value', true ) );

            return new MF_Campaigns_Type_5( $coupon, $skus, $values, $priority, $threshold_total );
        } elseif ( 'mf_type_set' == $discount_type ) {
			$values = json_decode( get_post_meta( $coupon->get_id(), 'mf_parts', true ) );

            return new MF_Campaigns_Type_Set( $coupon, $values, $priority );
        }

        return null;
    }
}
