<?php

defined( 'ABSPATH' ) || exit;

class MF_Campaigns_Type_1 implements MF_Campaigns_Discounter {
    /**
     * @param WC_Coupon $coupon
     * @param string[] $skus
     * @param int $x
     * @param int $y
     * @param int $priority
     * @param int $threshold_total
     */
    public function __construct( $coupon, $skus, $x, $y, $priority, $threshold_total ) {
        $this->coupon = $coupon;
        $this->skus = $skus;
        $this->x = intval( $x );
        $this->y = intval( $y );
        $this->priority = intval( $priority );
        $this->threshold_total = intval( $threshold_total );
    }

    public function get_name() {
        return __( 'Buy X amount and get Y discount', 'wpmf' );
    }

    public function get_coupon() {
        return $this->coupon;
    }

    public function get_priority() {
        return $this->priority;
    }

    public function is_applicable( $cart_total, $products ) {
        if ( $cart_total < $this->threshold_total ) {
            return false;
        }

        $applicable_products = $this->applicable_products( $products );

        if ( count( $applicable_products ) < $this->x ) {
            return false;
        }

        foreach ( $applicable_products as $applicable_product ) {
            $applicable_product->add_use();
        }

        return true;
    }

    public function get_discounted_products( $products, $cart_total ) {
        return $this->_get_discounted_products(
            $this->applicable_products( $products ),
            $cart_total
        );
    }

    protected function applicable_products( $products ) {
        $skus = $this->skus;
        return array_filter(
            $products,
            function ( $product ) use ( $skus ) {
                if ( $product->get_uses() > 0 ) {
                    return false;
                }
                return in_array( $product->get_sku(), $skus );
            }
        );
    }

    /**
     * @param MF_Campaigns_Product[] $products
     * @param int $maximum_allowed_discount
     * @return MF_Campaigns_Product[]
     */
    protected function _get_discounted_products( $products, $maximum_allowed_discount ) {
        if ( count( $products ) < $this->x ) {
            return [];
        }

        $discounted_products = array_splice( $products, -$this->x, $this->x ); // Remove x most expensive products
        if ( ! empty( $discounted_products ) ) {
            $maximum_allowed_discount -= $this->y;
            if ( $maximum_allowed_discount >= 0 ) {
                $discount = mf_wc_round( $this->y / $this->x, wc_get_rounding_precision() );
                foreach ( $discounted_products as $i ) {
                    $i->set_discount( $discount );
                }
            }
        }

        if ( empty( $products ) ) {
            return array_merge(
                $discounted_products
            );
        }

        return array_merge(
            $discounted_products,
            $this->_get_discounted_products( $products, $maximum_allowed_discount )
        );
    }

    /** @var WC_Coupon WooCommerce coupon for this discount */
    protected $coupon;
    /** @var string[] All skus valid for this discount */
    protected $skus;
    /** @var int Value X (amount needed to buy) */
    protected $x;
    /** @var int Value Y (amount with discount if X is fulfilled) */
    protected $y;
    /** @var int Priority */
    protected $priority;
    /** @var int Cart sum needed to activate this discount */
    protected $threshold_total;
}