<?php

defined( 'ABSPATH' ) || exit;

class MF_Campaigns_Type_4 implements MF_Campaigns_Discounter {
    /**
     * @param WC_Coupon $coupon
     * @param string[] $skus
     * @param object $values
     * @param int $priority
     * @param int $threshold_total
     */
    public function __construct( $coupon, $skus, $values, $priority, $threshold_total ) {
        $this->coupon = $coupon;
        $this->skus = $skus;
        $this->values = $values;
        $this->priority = intval( $priority );
        $this->threshold_total = intval( $threshold_total );
    }

    public function get_name() {
        return __( 'Special Offer', 'wpmf' );
    }

    public function get_coupon() {
        return $this->coupon;
    }

    public function get_priority() {
        return $this->priority;
    }

    public function is_applicable( $cart_total, $products ) {
        if ( $cart_total < $this->threshold_total ) {
            return false;
        }

        $applicable_products = $this->applicable_products( $products );

        if ( count( $applicable_products ) < 1 ) {
            return false;
        }

        foreach ( $applicable_products as $applicable_product ) {
            $applicable_product->add_use();
        }

        return true;
    }

    public function get_discounted_products( $products, $cart_total ) {
        return $this->_get_discounted_products(
            $this->applicable_products( $products ),
            $cart_total
        );
    }

    protected function applicable_products( $products ) {
        $skus = $this->skus;
        return array_filter(
            $products,
            function ( $product ) use ( $skus ) {
                if ( $product->get_uses() > 0 ) {
                    return false;
                }
                return in_array( $product->get_sku(), $skus );
            }
        );
    }

    /**
     * @param MF_Campaigns_Product[] $products
     * @param int $maximum_allowed_discount
     * @return MF_Campaigns_Product[]
     */
    protected function _get_discounted_products( $products, $maximum_allowed_discount ) {
        if ( empty( $products ) ) {
            return [];
        }

        foreach ( $products as $product ) {
            $discount = mf_wc_round(
                $product->get_price() - $this->get_value_of_sku( $product->get_sku() ),
                wc_get_rounding_precision()
            );

            if ( $maximum_allowed_discount >= $discount ) {
                $product->set_discount( $discount );
                $maximum_allowed_discount -= $discount;
            }
        }

        return $products;
    }

    /**
     * Gets value of sku from values array
     * 
     * @param string $sku
     * @return string
     */
    protected function get_value_of_sku( $sku ) {
        if ( property_exists( $this->values, $sku ) ) {
            return floatval( $this->values->{$sku} );
        }

        return 0.0;
    }

    /** @var WC_Coupon WooCommerce coupon for this discount */
    protected $coupon;
    /** @var string[] All skus valid for this discount */
    protected $skus;
    /** @var object SKU values */
    protected $values;
    /** @var int Priority */
    protected $priority;
    /** @var int Cart sum needed to activate this discount */
    protected $threshold_total;
}