<?php

defined( 'ABSPATH' ) || exit;

class MF_Sync_Receipts_Helper {
    /**
     * Get bookkeeping account number from product.
     * 
     * @param MF_Sync_Model_Account_Code[] $account_codes All available account codes.
     * @param WC_Product $product Product to get account code for.
     * @return MF_Sync_Model_Account_Code|false The account code, false if not found.
     */
    public static function bookkeeping_account_product( $account_codes, $product ) {
        $product_account_code_id = ( is_a( $product, 'WC_Product_Variation' ) )
            ? get_post_meta( $product->get_parent_id(), 'mf_account_code', true )
            : get_post_meta( $product->get_id(), 'mf_account_code', true );
        
        foreach ( $account_codes as $account_code ) {
            if ( $account_code->get_id() != $product_account_code_id ) {
                continue;
            }

            return $account_code->get_account_code();
        }

        return false;
    }

    /**
     * Gets bookkeeping account for payment method.
     * 
     * @param string $payment_method
     * @return string|bool Account on success, else false.
     */
    static public function bookkeeping_account_product_payment( $payment_method ) {
        $api_options = get_option(MOREFLO_API_URL);

        if ( ! isset( $api_options['bookkeeping_account_payments'] ) ) {
            return false;
        }

        return isset( $api_options['bookkeeping_account_payments'][ $payment_method ] )
            ? $api_options['bookkeeping_account_payments'][ $payment_method ]
            : false;
    }

    /**
     * Get tax percentage for order item of order.
     * 
     * @param WC_Order_Item_Product $order_item
     * @param WC_Order $order Order that the item refers to.
     * @return int
     */
	public static function get_order_row_tax_rate( $order_item, $order ) {
		$tax_percentage = 0;

        $tax_class = $order_item->get_tax_class();
        $args = array(
            'country'   => $order->get_billing_country(),
            'state'     => $order->get_billing_state(),
            'postcode'  => $order->get_billing_postcode(),
            'city'      => $order->get_billing_city(),
            'tax_class' => $tax_class,
        );

		if ( empty( $args['country'] ) ) {
            // Default to base
			$args['country']  = WC()->countries->get_base_country();
			$args['state']    = WC()->countries->get_base_state();
			$args['postcode'] = WC()->countries->get_base_postcode();
			$args['city']     = WC()->countries->get_base_city();
		}
        $tax_data = WC_Tax::find_rates( $args );

        if ( ! empty( $tax_data ) ) {
            foreach ($tax_data as $tax_rate) {
                $tax_percentage = $tax_rate['rate'];
            }
        }

		return $tax_percentage;
	}
    /**
     * Gets tax percentage for product in cart.
     * 
     * @param WC_Product $product
     * @param WC_Cart $cart
     * @return int
     */
	public static function get_cart_product_tax_rate( $product, $cart ) {
		$tax_percentage = 0;
		$wc_tax = new WC_Tax();

		$tax_class = $product->get_tax_class();

		$tax_data = $wc_tax->get_rates( $tax_class, $cart->get_customer() );

        if ( ! empty( $tax_data ) ) {
            foreach ($tax_data as $tax_rate) {
                $tax_percentage = $tax_rate['rate'];
            }
        }

		return $tax_percentage;
	}

    /**
     * Gets serial number of product.
     * 
     * @param WC_Product $product
     * @return string Serial number, empty string on not found.
     */
    public static function get_serial_number( $product ) {
        return ( is_a( $product, 'WC_Product_Variation' ) )
            ? get_post_meta( $product->get_parent_id(), '_custom_product_barcodes_field', true )
            : get_post_meta( $product->get_id(), '_custom_product_barcodes_field', true );
    }
}
