<?php

class MF_Sync_Model_Article extends MF_Sync_Receipts_Model implements JsonSerializable {
    /** @var int */
    protected $OrderIndex;
    /** @var string */
    protected $ArticleNumber;
    /** @var string */
    protected $Name;
    /** @var int */
    protected $Quantity;
    /** @var float */
    protected $UnitPrice;
    /** @var float */
    protected $UnitPriceWithoutDiscounts;
    /** @var int */
    protected $VatRate;
    /** @var float */
    protected $TotalPriceIncVat;
    /** @var float */
    protected $TotalPriceExVat;
    /** @var float */
    protected $TotalVat;
    /** @var string */
    protected $BookkeepingAccount;
    /** @var array */
    protected $Discounts;
    /** @var array */
    protected $Attributes;
    
    /** @var array */
    protected $taxes_totals;
    /** @var string */
    protected $type;

    /**
     * Gets total tax for specific rate.
     * 
     * @param string $rate_id
     * @return float|null Total tax on success, else null.
     */
    public function get_tax_total( $rate_id ) {
        return isset( $this->taxes_totals['total'][ $rate_id ] )
            ? $this->taxes_totals['total'][ $rate_id ]
            : null;
    }

    /**
     * @return float
     */
    public function get_total_price_inc_vat() {
        return $this->TotalPriceIncVat;
    }

    /**
     * @return float
     */
    public function get_total_price_ex_vat() {
        return $this->TotalPriceExVat;
    }

    /**
     * @return string
     */
    public function get_bookkeeping_account() {
        return $this->BookkeepingAccount;
    }

    /**
     * @return string
     */
    public function get_type() {
        return $this->type;
    }

    /**
     * @return string
     */
    public function get_sku() {
        return $this->ArticleNumber;
    }

    /**
     * @return string
     */
    public function get_name() {
        return $this->Name;
    }

    /**
     * @return int
     */
    public function get_quantity() {
        return $this->Quantity;
    }

    /**
     * @return float
     */
    public function get_unit_price() {
        return $this->UnitPrice;
    }

    /**
     * @return float
     */
    public function get_unit_price_without_discounts() {
        return $this->UnitPriceWithoutDiscounts;
    }

    /**
     * @return int
     */
    public function get_vat_rate() {
        return $this->VatRate;
    }

    /**
     * @return float Discount as a float. Must be rounded before use.
     */
    public function get_total_discount() {
        return array_reduce(
            $this->Discounts,
            function ( $carry, $discount ) {
                if ( 'amount' == $discount['DiscountType'] ) {
                    return $carry + $discount['Amount'];
                }

                return 0.0;
            },
            0.0
        );
    }

    public function jsonSerialize() {
        return [
            'OrderIndex' => $this->OrderIndex,
            'ArticleNumber' => $this->ArticleNumber,
            'Name' => $this->Name,
            'Quantity' => $this->Quantity,
            'UnitPrice' => $this->UnitPrice,
            'VatRate' => $this->VatRate,
            'TotalPriceIncVat' => $this->TotalPriceIncVat,
            'TotalPriceExVat' => $this->TotalPriceExVat,
            'TotalVat' => $this->TotalVat,
            'Discounts' => $this->Discounts,
            'Attributes' => $this->Attributes,
        ];
    }
}
