<?php

defined( 'ABSPATH' ) || exit;

class MF_Sync_Receipts {
    public function __construct() {
        add_filter( 'moreflo_sync_before_order_sync', array( $this, 'before_order_sync' ), 10, 3 );
        add_filter( 'moreflo_sync_before_order_refund_sync', array( $this, 'before_order_refund_sync' ), 10, 3 );
    }

    /**
     * Filters payment status based on receipt status.
     *  - Needs Receipt AND FullyPaid => Set payment status to NotPaid
     * 
     * @param array $mf_order Array to send to Apps
     * @param WC_Order $order WooCommerce order or refund order.
     * @param string $method Method used to sync order, 'add' or 'update'.
     */
    public function before_order_sync( $mf_order, $order, $method ) {
        // Is receipts enabled?
        if ( ! moreflo_sync_receipts_enabled() ) {
            return $mf_order;
        }

        // Needs Receipt AND FullyPaid => Set payment status to NotPaid
        if ( 'needs-receipt' == get_post_meta( $order->get_id(), '_mf_receipt_status', true )
            && 'FullyPaid' == $mf_order['PaymentStatus']
        ) {
            $mf_order['PaymentStatus'] = 'NotPaid';
        }

        return $mf_order;
    }

    /**
     * Filters payment status based on receipt status.
     *  - Has not send receipt AND FullyPaid => Set payment status to NotPaid
     * 
     * @param array $mf_order Array to send to Apps
     * @param WC_Order_Refund $order WooCommerce order or refund order.
     * @param WC_Order $order Parent order for refund.
     */
    public function before_order_refund_sync( $mf_order, $order, $parent_order ) {
        // Is receipts enabled?
        if ( ! moreflo_sync_receipts_enabled() ) {
            return $mf_order;
        }

        // Make sure parent order has sent receipt
        if ( 'receipt-sent' != get_post_meta( $parent_order->get_id(), '_mf_receipt_status', true ) ) {
            return $mf_order;
        }

        // Has not send receipt AND FullyPaid => Set payment status to NotPaid
        if ( 'receipt-sent' != get_post_meta( $order->get_id(), '_mf_receipt_status', true )
            && 'FullyPaid' == $mf_order['PaymentStatus']
        ) {
            $mf_order['PaymentStatus'] = 'NotPaid';
        }

        return $mf_order;
    }
}

new MF_Sync_Receipts();
