<?php
define('ENABLED', 'enabled');
define('PRODUCT_TYPE', 'product_type');
define('SIMPLE', 'simple');
define('WCBOOKINGBLOCKCOST', '_wc_booking_block_cost');
define('MANAGESTOCK', '_manage_stock');
define('STOCKSTATUS', '_stock_status');
define('DOWNLOADABLE', '_downloadable');
define('VIRTUAL', '_virtual');
define('OUTOFSTOCK', 'outofstock');
define('INSTOCK', 'instock');
define('PRODUCTVISIBILITY', 'product_visibility');
class MfArticles
{
	private $api_options;
	private $is_product_title_active;
	private $is_price_active;
	private $is_pic_active;
	private $is_mf_tax_active;

	function __construct() {
		$this->api_options = get_option(MOREFLO_API_URL);
		$this->is_product_title_active = ( isset( $this->api_options['is_product_title_active'] ) )
			? $this->api_options['is_product_title_active']
			: '';
		$this->is_price_active = ( isset( $this->api_options[ISPRICEACTIVE] ) )
			? $this->api_options[ISPRICEACTIVE]
			: '';
		$this->is_pic_active = isset( $this->api_options['is_pic_active'] )
			? $this->api_options['is_pic_active']
			: '';
		$this->is_mf_tax_active = isset( $this->api_options['is_tax_class_active'] )
			? $this->api_options['is_tax_class_active']
			: '';
	}
	

    function add($product)
    {

        try {
	        $product_obj = new WC_Product();
	        $product_obj->set_name($product['Name']);
	        $product_obj->set_sku($product['Sku']);
			if ( isset( $this->api_options['sync_publish_products_status'] )
				&& 'enabled' == $this->api_options['sync_publish_products_status']
				&& $product[ARTICLETYPE] == TYPEGOODS
			) {
				$product_obj->set_status($product[ISACTIVEARTICLE]);
			} else {
				$product_obj->set_status(ARTICLEDRAFT);
			}
	        $product_obj->set_featured('no');
	        $product_obj->set_catalog_visibility('visible');

			$mf_wc_tax_rules = get_option('mf_wc_tax_rules');
			$woocommerce_calc_taxes_enable = get_option('woocommerce_calc_taxes');
	        if ($this->is_price_active == ENABLED) {
				$woocommerce_prices_include_tax = get_option('woocommerce_prices_include_tax');
				if ($this->is_mf_tax_active == ENABLED && $woocommerce_prices_include_tax == 'no' && $woocommerce_calc_taxes_enable == 'yes') {
					//Calculate amount
					$tax_rate = $mf_wc_tax_rules[$product[ACCOUNTCODEID]][1];
					$tax_amount = $product[RETAILPRICE] - ( $product[RETAILPRICE] / ( ( $tax_rate / 100 ) + 1 ) );
					$reCalPrice = $product[RETAILPRICE] - $tax_amount;
					$product_obj->set_regular_price($reCalPrice);
				}else{
					$product_obj->set_regular_price($product[RETAILPRICE]);
				}
	        }

			if ( isset( $this->api_options['sync_article_description'] ) && 'enabled' == $this->api_options['sync_article_description'] ) {
				$product_obj->set_description( $product['ArticleTextHtml'] );
			}

			$new_product_id = $product_obj->save();

			update_post_meta($new_product_id, MF_ACCOUNT_CODE, $product[ACCOUNTCODEID]);

		
			

			if ($this->is_mf_tax_active == ENABLED) {
				update_post_meta($new_product_id, '_tax_status', 'taxable');
				update_post_meta($new_product_id, '_tax_class', $mf_wc_tax_rules[$product[ACCOUNTCODEID]][0]);
			}
			
			if(isset( $product[BARCODE]) && $product[BARCODE]){
				add_post_meta($new_product_id, '_custom_product_barcodes_field', $product[BARCODE]);//Adding barcode 19-02-2019
			}
			if(isset( $product['BarcodeAlternatives']) && $product['BarcodeAlternatives']) {
				// Loopa igenom alla alternativa barcodes och lägg i listan
				foreach ( $product['BarcodeAlternatives'] as $barcode ) {

					if(!empty($barcode['Barcode'])) {
						$altBarCode =  $barcode['Barcode'];
						add_post_meta($new_product_id, '_custom_product_barcodes_field', $altBarCode);
					}
	
				}
			}
			/**
			*Handling simple products
			*/
            if ($product[ARTICLETYPE] == TYPEGOODS) {
	            wp_set_object_terms($new_product_id, SIMPLE, PRODUCT_TYPE);

	            $this->update_stock($new_product_id, $product[QUANTITY], $product[ISSTOCKITEM]);
            }

			/**
			*Handling bookable products
			*/
			
            if($product[ARTICLETYPE] == TYPESERVICE) {
	            if (mf_is_wc_appointments_active()) {
					wp_set_object_terms($new_product_id, 'appointment', 'product_type', false);
					if($new_product_id)
					{
						$this->bookableProductAttr($new_product_id, $product);
						update_post_meta($new_product_id, '_wc_appointment_staff_assignment', 'customer');
					}
	            } else {
		            wp_set_object_terms($new_product_id, 'booking', 'product_type', false);

		            if ($this->is_price_active == "enabled") {
			            update_post_meta($new_product_id, '_wc_booking_block_cost', $product['RetailPrice']);
		            }

	            }

	            update_post_meta($new_product_id, DOWNLOADABLE, 'no' );
	            update_post_meta($new_product_id, VIRTUAL, 'yes');
	            update_post_meta($new_product_id, MANAGESTOCK, 'no');
	            update_post_meta($new_product_id, STOCKSTATUS, INSTOCK);

			}
			MfLog::log(DEBUG, $new_product_id.'product productype : '.$product[ARTICLETYPE]);
			/**
			 * Image upload integration 06-10-2020
			 */
			if(($this->is_pic_active == ENABLED) && $product['Files']){
				$this->upload_images($new_product_id, $product['Files']);
			}

			do_action( 'moreflo_sync_articles_add', $new_product_id, $product );

			/**
			 * Save GroupId
			 */
			$this->updateProductGroup($new_product_id, $product['GroupId']);

	        return $new_product_id;

        } catch (Exception $e) {
	        MfLog::log(ERROR,
		        'Exception occurred while adding the product to the site: ' .$product['Sku']. ' ' . $e->getMessage()
	        );
        }

    }

	function update($product_id, $product)
	{

		try {

			$product_obj = wc_get_product($product_id);

			if($product_obj instanceof WC_Product) {
				
				if($this->is_product_title_active == ENABLED)
				{
					$product_obj->set_name($product['Name']);
				}
				if ( isset( $this->api_options['sync_publish_products_status'] )
					&& 'enabled' == $this->api_options['sync_publish_products_status']
					&& $product[ARTICLETYPE] == TYPEGOODS
				) {
					$product_obj->set_status($product[ISACTIVEARTICLE]);
				}
				
				$product_type = WC_Product_Factory::get_product_type($product_id);

				$mf_wc_tax_rules = get_option('mf_wc_tax_rules');
				$woocommerce_calc_taxes_enable = get_option('woocommerce_calc_taxes');
				if ($this->is_price_active == ENABLED) {
					$productPrice = $product[RETAILPRICE];
					$woocommerce_prices_include_tax = get_option('woocommerce_prices_include_tax');
					if ($this->is_mf_tax_active == ENABLED && $woocommerce_prices_include_tax == 'no' && $woocommerce_calc_taxes_enable == 'yes') {
						//Calculate amount
						$tax_rate = $mf_wc_tax_rules[$product[ACCOUNTCODEID]][1];
						$tax_amount = $productPrice - ( $productPrice / ( ( $tax_rate / 100 ) + 1 ) );
						$productPrice = $productPrice - $tax_amount;
					}

					if($product_type == BOOKING && $this->is_price_active == ENABLED)
					{
						update_post_meta($product_id, WCBOOKINGBLOCKCOST, $productPrice);//Set regular price from Retail price
					}

					if($product_type != BOOKING && $this->is_price_active == ENABLED)
					{
						$product_obj->set_regular_price($productPrice);
					}
				}


				if($product_obj instanceof WC_Product_Variation) {

					update_post_meta($product_obj->get_parent_id(), MF_ACCOUNT_CODE, $product[ACCOUNTCODEID]);
					/**
					 * To do get main image and add to thumbnail and rest of image save it main products
					 */
					
				} else {
					update_post_meta($product_id, MF_ACCOUNT_CODE, $product[ACCOUNTCODEID]);

					if(($this->is_pic_active == ENABLED) && $product['Files']){
						$this->upload_images($product_id, $product['Files']);
					}	
				}
				// Ta bort alla för vi vet inte vem som är ny.

				if ( get_post_meta($product_id, '_custom_product_barcodes_field', false) ) {
					delete_post_meta($product_id, '_custom_product_barcodes_field');
				}
				
				if(isset( $product[BARCODE]) && $product[BARCODE]){
					add_post_meta($product_id, '_custom_product_barcodes_field', $product[BARCODE]); 
				} 

				if(isset( $product['BarcodeAlternatives']) && $product['BarcodeAlternatives']) {
					// Loopa igenom alla alternativa barcodes och lägg i listan
					foreach ( $product['BarcodeAlternatives'] as $barcode ) {
	                     
						if(empty($barcode['Barcode']) == false) {
							
							$altBarCode =  $barcode['Barcode'];
							
							MfLog::log(DEBUG, 'Alternativ streckkod=' . $altBarCode);
							add_post_meta($product_id, '_custom_product_barcodes_field', $altBarCode);
						}
		
					}
				}


				if ($this->is_mf_tax_active == ENABLED && isset( $product[ACCOUNTCODEID] ) && isset( $mf_wc_tax_rules[$product[ACCOUNTCODEID]] )) {
					$mf_wc_tax_rules = get_option('mf_wc_tax_rules');
					update_post_meta($product_id, '_tax_status', 'taxable');			
					update_post_meta($product_id, '_tax_class', $mf_wc_tax_rules[$product[ACCOUNTCODEID]][0]);
				}

				if ( isset( $this->api_options['sync_article_description'] ) && 'enabled' == $this->api_options['sync_article_description'] ) {
					$product_obj->set_description( $product['ArticleTextHtml'] );
				}

				$product_obj->save();

				$serviceTypes = [BOOKING, 'appointment'];

				if ($product[ARTICLETYPE] == TYPEGOODS && ! in_array( $product_type, ['simple', 'composite', 'variable'] )) {
					$object_terms = wp_get_object_terms( $product_id, 'product_type' );
					if ( in_array( 'booking', $object_terms ) ) {
						wp_remove_object_terms( $product_id, 'booking', 'product_type' );
					}
					if ( in_array( 'appointment', $object_terms ) ) {
						wp_remove_object_terms( $product_id, 'appointment', 'product_type' );
					}
					wp_set_object_terms($product_id, SIMPLE, PRODUCT_TYPE);

					delete_post_meta( $product_id, DOWNLOADABLE, 'no');
					delete_post_meta( $product_id, VIRTUAL, 'yes');
					delete_post_meta( $product_id, MANAGESTOCK, 'no');
					delete_post_meta( $product_id, STOCKSTATUS, INSTOCK);

					MfLog::log(DEBUG, 'Product type change to Simple');
				}
				if($product[ARTICLETYPE] == TYPESERVICE && !in_array($product_type, $serviceTypes)) {
					wp_remove_object_terms( $product_id, 'simple', 'product_type' );
					if (mf_is_wc_appointments_active()) {
						wp_set_object_terms($product_id, 'appointment', 'product_type', false);
					} else {
						wp_set_object_terms($product_id, 'booking', 'product_type', false);
						if($this->is_price_active == ENABLED){
							update_post_meta($product_id, '_wc_booking_block_cost', $product[RETAILPRICE]);
						}
					}
					update_post_meta($product_id, DOWNLOADABLE, 'no' );
					update_post_meta($product_id, VIRTUAL, 'yes');
					update_post_meta($product_id, MANAGESTOCK, 'no');
					update_post_meta($product_id, STOCKSTATUS, INSTOCK);

					MfLog::log(DEBUG, 'Product type change to Not simple(appointment/booking/subscription)');
				}
				if($product[ARTICLETYPE] == TYPESERVICE) {					
					update_post_meta($product_id, WCBOOKINGBLOCKCOST, $product[RETAILPRICE]);//Set regular price from Retail price

					if (mf_is_wc_appointments_active()) {
						$this->bookableProductAttr($product_id, $product);
					}
				}

				$this->update_stock($product_id, $product[QUANTITY], $product[ISSTOCKITEM], $product_obj->get_parent_id());			

				do_action( 'moreflo_sync_articles_update', $product_id, $product );

				/**
				 * Save GroupId
				 */
				$this->updateProductGroup($product_id, $product['GroupId']);

			}

		} catch (Exception $e) {
			MfLog::log(ERROR,
				'Exception occurred while updating the product to the site: ' . $product['Sku'] . ' ' . $e->getMessage()
			);
		}

	}

	public function updateOnlyStock( $product_id, $product ) {
		try {
			$product_obj = wc_get_product($product_id);

			if( $product_obj instanceof WC_Product ) {
				$this->update_stock($product_id, $product[QUANTITY], $product[ISSTOCKITEM], $product_obj->get_parent_id());
				do_action( 'moreflo_sync_articles_update_only_stock', $product_id, $product );
			}
		} catch (Exception $e) {
			MfLog::log(
				ERROR,
				'Exception occurred while updating the product to the site: ' . $product['Sku'] . ' ' . $e->getMessage()
			);
		}
	}

	/**
	 * Updates product group to Apps group if it exists
	 * 
	 * @param int $product_id
	 * @param string $mf_group_id
	 */
	private function updateProductGroup($product_id, $mf_group_id) {
		if ( ! moreflo_sync_groups_enabled() ) {
			return;
		}
		
		update_post_meta($product_id, 'mf_group_id', $mf_group_id);
		$terms = [];
		if ( ! empty( $mf_group_id ) ) {
			$terms = get_terms([
				'taxonomy' => 'product_cat',
				'meta_query' => [
					[
						'key' => 'mf_group_id',
						'value' => $mf_group_id,
					],
				],
				'fields' => 'ids',
				'number' => 1,
				'hide_empty' => false,
			]);
		}
		wp_set_object_terms($product_id, $terms, 'product_cat');
	}

	private function bookableProductAttr($product_id, $product)
	{

		update_post_meta($product_id, '_wc_appointment_duration_unit', $product['DefaultDurationUnit']);
		update_post_meta($product_id, '_wc_appointment_duration', $product['DefaultDuration']);
		update_post_meta($product_id, '_wc_appointment_interval_unit', $product['IntervalTimeUnit']);
		update_post_meta($product_id, '_wc_appointment_interval', $product['IntervalTime']);
		update_post_meta($product_id, '_wc_appointment_padding_duration_unit', $product['PaddingTimeUnit']);
		update_post_meta($product_id, '_wc_appointment_padding_duration', $product['PaddingTime']);
		update_post_meta($product_id, '_wc_appointment_min_date_unit', $product['LeadTimeUnit']);
		update_post_meta($product_id, '_wc_appointment_min_date', $product['LeadTime']);
		update_post_meta($product_id, '_wc_appointment_max_date_unit', $product['SchedulingWindowUnit']);
		update_post_meta($product_id, '_wc_appointment_max_date', $product['SchedulingWindow']);
		update_post_meta($product_id, '_wc_appointment_user_can_cancel', $product['ClientCancellationPossible']);
		update_post_meta($product_id, '_wc_appointment_requires_confirmation', $product['RequiresStoreConfirmation']);
		update_post_meta($product_id, '_wc_appointment_cancel_limit_unit', "hour");
		update_post_meta($product_id, '_wc_appointment_cancel_limit', 12);

		if($product['ArticleBookableType']){
			update_post_meta($product_id, '_wc_mf_bookable_type', $product['ArticleBookableType']);
		}
		
		$my_post = array(
			'ID'           => $product_id,
			'post_content' => $product['DetailedDescriptionHtml'],
		);
		wp_update_post( $my_post );
	}

	function update_stock($product_id, $quantity, $manage_stock = null, $parent_id = null)
	{
		if($product_id > 0) {

			if($manage_stock !== null) {
				update_post_meta($product_id, MANAGESTOCK, ($manage_stock ? 'yes' : 'no'));
			}

			if(get_post_meta($product_id, MANAGESTOCK, true) === 'yes') {
				update_post_meta($product_id, '_stock', $quantity);

				/* Handling the stock status like WC3 https://stackoverflow.com/a/51940564 */
				if ( in_array( get_post_meta( $product_id, STOCKSTATUS, true ), [ 'onbackorder' ] ) ) {
					wp_remove_object_terms($product_id, OUTOFSTOCK, PRODUCTVISIBILITY);
				} elseif($quantity > 0) {
					update_post_meta($product_id, STOCKSTATUS, INSTOCK);
					wp_remove_object_terms($product_id, OUTOFSTOCK, PRODUCTVISIBILITY);
				} else {
					update_post_meta($product_id, STOCKSTATUS, OUTOFSTOCK);
					wp_set_post_terms( $product_id, OUTOFSTOCK, PRODUCTVISIBILITY, true );
				}

				wc_delete_product_transients($product_id);
			} else {
				// Always set product to in stock when not manging stock
				update_post_meta($product_id, STOCKSTATUS, INSTOCK);
			}
		}
	}

	function upload_images($product_id, $images)
	{
		MfLog::log(DEBUG,'upload images:');
		/**
		 * Todo
		 * Stop creating multiple images a for products which is stop force sync creting multiple image generated */
		$imageGallery = array();
		foreach ($images as $image){
			$image_url        = $image['URL']; // Define the image URL here
			$image_name       = $image['Name'];
			$image_type       = $image['ContentType'];
			$feature_image    = $image['IsMainFile'];
			$tmp			  = get_temp_dir().$image_name; // temp folder locations and file name

			// If file is already imported, skip download
			$already_exists = get_posts([
				'post_type' => 'attachment',
				'posts_per_page' => 1,
				'ignore_sticky_posts' => true,
				'meta_query' => [
					'relation' => 'AND',
					[
						'key' => '_mf_sync_image_id',
						'value' => $image['Id'],
					],
					[
						'key' => '_mf_sync_image_name',
						'value' => $image['Name'],
					],
				],
				'fields' => 'ids',
			]);

			if ( ! empty( $already_exists ) ) {
				MfLog::log(DEBUG,'Image already exists, skipping download of image ('. $image['Id'] .')');

				if ( $feature_image ) {
					set_post_thumbnail( $product_id, $already_exists[0] );
				} else {
					array_push( $imageGallery, $already_exists[0] );
				}

				wp_update_post([
					'ID' => $already_exists[0],
					'post_content' => $image['Description'],
				]);
				continue;
			}

			$compatiableTypes = ['image/jpeg', 'image/png', 'image/gif'];
			if (!in_array($image_type, $compatiableTypes ) )
			{
				continue;
			}
			$api = new MfApi();
			$image_data = $api->get_image_file($image_url, $image_type);

			if($image_data){
				require_once(ABSPATH . 'wp-admin/includes/media.php');
				require_once(ABSPATH . 'wp-admin/includes/file.php');
				require_once(ABSPATH . 'wp-admin/includes/image.php');

				// Download file to temp location
				$tmpdl = file_put_contents($tmp, $image_data);

				// Set variables for storage
				$file_array['name'] = $image_name;
				$file_array['tmp_name'] = $tmp;	

				if ( is_wp_error( $tmpdl ) ) {
					@unlink($file_array['tmp_name']);
					MfLog::log(ERROR,'Error on download image from url :'.$image_url);
					continue;
				}else{
					MfLog::log(DEBUG,'Sucscessfuly downloaded image from url :'.$image_url);
				}
				/**Below verify the image name and will be delete and add as new image. Which stop increase file bandwidth */
				// $attachment_post_id;
				// $attachment_args = array(
                //     'posts_per_page' => 1,
                //     'post_type'      => 'attachment',
                //     'name'           => $image_name
                // );
                // $attachment_posts = new WP_Query($attachment_args);
                // if($attachment_posts->have_posts()){
				// 	while($attachment_posts->have_posts()) {
				// 		$attachment_posts->the_post();
				// 		$attachment_post_id = get_the_ID() ;
				// 	 }
				// }
				// if(!empty($attachment_post_id)){
				// 	MfLog::log(DEBUG,'Image name already existing deleting image : '.$image_name);
				// 	wp_delete_attachment($attachment_post_id);
				// }

				MfLog::log(DEBUG,'start image upload to wordpress');
				$imageid = media_handle_sideload( $file_array, $product_id, $image_name, [
					'post_content' => $image['Description'],
				] );

				if ( is_wp_error($imageid) ) {
					@unlink($file_array['tmp_name']);
					MfLog::log(ERROR,'Error on upload to wordpress image :'.$imageid->get_error_message());
					continue;
				}else{
					MfLog::log(DEBUG,'Sucscessfuly upload image to product '.$product_id);
					update_post_meta( $imageid, '_mf_sync_image_id', $image['Id'] );
					update_post_meta( $imageid, '_mf_sync_image_name', $image['Name'] );
				}

				if($feature_image){
					set_post_thumbnail($product_id, $imageid);
				}else{
					array_push($imageGallery, $imageid);
				}
				unset($imageid);
			}else{
				MfLog::log(ERROR,'Error on download image from url:'.$image_url.'.Might be due article name containing spaces.EX: /articles/Evicio+test/');
			}
		}
		if(!empty($imageGallery)){
			update_post_meta( $product_id, '_product_image_gallery', implode(',', $imageGallery));
		}
		MfLog::log(DEBUG,'upload images End:');
	}	

	function productToArticle($product) {

    	if($product instanceof WC_Product) {

    		$product_id = $product->get_id();
		    $product_type = WC_Product_Factory::get_product_type($product_id);
		    
		    $manage_stock = get_post_meta($product_id,MANAGESTOCK, true) == 'yes' ? true : false;

		    if($product instanceof WC_Product_Variation) {
			    $account_code = get_post_meta($product->get_parent_id(),MF_ACCOUNT_CODE, true);
		    } else {
			    $account_code = get_post_meta($product_id,MF_ACCOUNT_CODE, true);
		    }

		    $quantity = $product->get_stock_quantity();

		    $article = array(
			    ARTICLENAME => $product->get_name(),
			    ARTICLENUMBER => $product->get_sku(),
			    UNIT => 'St',
			    //ISACTIVE => false,/**This is commenting out to stop post draft product as inactive article to MoreFlo. Request #8698 */
			    UPDATEONEXISTING => true,
			    ISWEBSHOPARTICLE => true,
			    ACCOUNTCODEID => $account_code,
			);

			if($this->is_product_title_active != ENABLED)
			{
				unset($article[ARTICLENAME]);
			}


		    if($product_type === BOOKING) {
			    $regular_price = get_post_meta($product_id, WCBOOKINGBLOCKCOST, true);
		    } else {
			    $regular_price = $product->get_regular_price();
			}
			

			if ($this->is_price_active == ENABLED) {
				$article[RETAILPRICE] = $regular_price;
			}

		    if($product->get_status() == ARTICLEPUBLISH) {
			    $article[ISACTIVE] = true;
		    }

		    if($manage_stock && $quantity !== null) {
			    $article[QUANTITY] = $quantity;
		    }

		    if($manage_stock) {
			    $article[ISSTOCKITEM] = true;
		    } else {
			    $article[ISSTOCKITEM] = false;
		    }

		    switch ($product_type) {
			    case SIMPLE: $article[ARTICLETYPE] = TYPEGOODS; break;
			    case VARIABLE: $article[ARTICLETYPE] = TYPEGOODS; break;
			    case APPOINTMENT: $article[ARTICLETYPE] = TYPESERVICE; break;
			    case BOOKING: $article[ARTICLETYPE] = TYPESERVICE; break;
			    default: $article[ARTICLETYPE] = TYPEGOODS;
		    }

			if ( TYPEGOODS == $article[ ARTICLETYPE ] && ! get_post_meta( $product_id, MF_HASH, true ) ) {
				// Set new products as Ackumulative
				$article['Ackumulative'] = true;
			}

		    return $article;

	    } else {
		    MfLog::log(ERROR, 'This product is not an instance of WCProduct');
	    }

    	return false;

	}

	function variantToArticle($product, $variant_id) {

		$variant = wc_get_product($variant_id);

		if($variant instanceof WC_Product_Variation) {

			$product_id = $product->get_id();
			$manage_stock = get_post_meta($variant_id,MANAGESTOCK, true);
			$account_code = get_post_meta($product_id,MF_ACCOUNT_CODE, true);
			$quantity = $variant->get_stock_quantity();

			$article = array(
				ARTICLENAME => $this->get_variation_name( $product, $variant ),
				ARTICLENUMBER => $variant->get_sku( 'edit' ),
				UNIT => 'St',
				//ISACTIVE => false,/**This is commenting out to stop post draft product as inactive article to MoreFlo. Request #8698 */
				UPDATEONEXISTING => true,
				ISWEBSHOPARTICLE => true,
				ACCOUNTCODEID => $account_code,
			);

			if($this->is_product_title_active != ENABLED)
			{
				unset($article[ARTICLENAME]);
			}


			$regular_price = $variant->get_regular_price();

			if ($this->is_price_active == ENABLED) {
				$article[RECOMMENDEDPRICE] = $regular_price; //If sale price has not been set in woocommerce, both recommonded and retail prices will be equal.
				$article[RETAILPRICE] = $regular_price;
			}

			if($variant->get_status() == ARTICLEPUBLISH) {
				$article[ISACTIVE] = true;
			}

			if(($manage_stock == 'yes') && $quantity !== null) {
				$article[QUANTITY] = $quantity;
			}

			if($manage_stock == 'yes') {
				$article[ISSTOCKITEM] = true;
			} else {
				$article[ISSTOCKITEM] = false;
			}

			$article[ARTICLETYPE] = TYPEGOODS;

			if ( TYPEGOODS == $article[ ARTICLETYPE ] && ! get_post_meta( $variant_id, MF_HASH, true ) ) {
				// Set new products as Ackumulative
				$article['Ackumulative'] = true;
			}

			return $article;

		} else {
			MfLog::log(ERROR, 'This product is not an instance of WC_Product_Variation');
		}

		return false;

	}

	/**
	 * Get the variation name. Will cut names longer than 60 characters
	 * 
	 * @param WC_Product $product
	 * @param WC_Product_Variation $variation
	 */
	private function get_variation_name( $product, $variation ) {
		$full_name = $product->get_name() .' - '. wc_get_formatted_variation( $variation, true, false );

		return mb_substr( $full_name, 0, 60 );
	}
}
