<?php

namespace Moreflo;

use MF_Sync_Receipts_Factory;
use WC_Order_Query;
use WC_Product;
use WC_Order;
use MfLog;
use WC_Product_Factory;
use WC_Tax;

class Order {

	/** @var WC_Order|null */
	private $order;
	private $product_types;

	function __construct($order) {

		if($order instanceof WC_Order) {
			$this->order = $order;
		} else {
			$this->order = null;
		}

		$this->product_types = array();

	}

	function order_to_moreflo_order() {

		if ( !is_null($this->order) ) {

			$order_no = $this->order->get_order_number();

			$mf_order = array(
				'ExternalReference' => 'MFS-'.$order_no,
				UPDATEONEXISTING  => true,
				'OurReference'      => $order_no,
				'YourOrderNo'         => $order_no,
			);

			$mf_service_order = array(
				ID => $order_no,
				'Type' => BOOKING,
			);

			$customer = $this->get_customer();
			$billing_address = $this->get_billing_address();
			$shipping_address = $this->get_shipping_address();
			$order_rows = $this->get_order_rows();
			$order_notes = $this->order->get_customer_note();
			$order_type = $this->get_order_type();
			$booking_data = $this->get_booking_data();
			$mf_order_data = get_post_meta($order_no, 'mf_order');

			$mf_resource = [];
			if(is_array($booking_data) && is_array($booking_data['staff_id']) && !empty($booking_data['staff_id'])){
				$resource = get_user_by('ID', $booking_data['staff_id'][0]);
				$mf_resource = array(
					'BookableResourceId' => $resource->user_login,
				);
			}
			unset($booking_data['staff_id']);

			//Saving the order type as meta
			update_post_meta($order_no, MF_ORDER_TYPE, $order_type);

			if(!empty($mf_order_data['mf_order_no'])) {
				$mf_order['Number'] = $mf_order_data['mf_order_no'];
			}

			if(!empty($customer)) {
				if(empty($customer['CustomerNumber']))
				{
					$customer['CustomerNumber'] = $this->get_customer_number($customer);
				}
				$mf_order['Customer'] = $customer;
			}

			if(!empty($billing_address)) {
				$mf_order['BillingAddress'] = $billing_address;
			}

			if(!empty($shipping_address)) {
				$mf_order['ShippingAddress'] = $shipping_address;
			}

			if(!empty($order_notes)) {
				$mf_order['Note'] = $order_notes;
			}

			$mf_order['PaymentOption'] = $this->get_order_pay_method();
			$mf_order['PaymentStatus'] = $this->get_order_pay_status();

			if($this->getPayMethod() == 'moreflo_invoice')
			{
				$mf_order['PaymentReference'] =  'Invoice';
			}

			$mf_order['State'] = $this->get_order_status($order_type);

			$articles = [];
			if($order_type == 'SERVICE_ORDER' && is_array($booking_data) && !empty($booking_data)) {
				$mf_order['Customer']['ReferenceNo'] = 0;
				if(isset($mf_order['Note']))
				{
					$mf_order['InternNote'] = $mf_order['Note'];
					unset($mf_order['Note']);
				}

				foreach($order_rows as $order_row){
					if(isset($order_row['UnitAmountInclVatExclDiscount']) || isset($order_row['VatRate'])) {
						$order_row['RetailPrice'] = $order_row['UnitAmountInclVatExclDiscount'];
						$order_row['Vat'] = $order_row['VatRate'];
						unset($order_row['UnitAmountInclVatExclDiscount']);
						unset($order_row['VatRate']);
					}
					array_push($articles, $order_row);
				}
				$mf_order['Articles'] = $articles;
				$mf_order = array_merge($mf_order, $mf_resource, $mf_service_order, $booking_data);
				
				// $mf_order = array_merge($mf_order, $mf_service_order, $booking_data);
				// $mf_order['Articles'] = $order_rows;
			} else {
				$mf_order['Rows'] = $order_rows;
			}

			$mf_order = apply_filters( 'moreflo_sync_order_to_moreflo_order', $mf_order, $this->order->get_id() );

			return $mf_order;

		} else {
			MfLog::log( ERROR, 'This order is not an instance of WC_Order' );
		}

		return false;

	}

	public function refund_order_to_moreflo_order( $mf_order, $parent_order_id ) {
		$order_no = $this->order->get_order_number();
		
		$mf_order['ExternalReference'] = 'MFS-'. $order_no;
		$mf_order['OurReference'] = $order_no;
		$mf_order['YourOrderNo'] = $order_no;
		$mf_order['Rows'] = $this->get_refund_order_rows( $parent_order_id );
		$mf_order['PaymentOption'] = $this->get_order_pay_method();
		$mf_order['PaymentStatus'] = $this->get_order_pay_status();
		$mf_order['State'] = $this->get_order_status( 'refund' );
		
		return $mf_order;
	}

	/**
	 * @param WC_Order_Refund $order
	 * @param bool $restock_items
	 * @return MF_Sync_Model_Receipt|WP_Error|bool
	 */
	public function order_to_moreflo_receipt_refund( $order_refund, $restock_items ) {
		$api_options = get_option('moreflo_api_url');
		$mf_account_codes = get_option('mf_account_ids');

		// Is receipts enabled?
		if ( ! moreflo_sync_receipts_enabled() ) {
			return false;
		}

		// Company corp no, name and bookable account set?
		if ( ! $api_options['mf_company_corp_no'] || ! $api_options['mf_company_name'] ) {
			return false;
		}

		// Only create for orders that has sent receipts
		if ( 'receipt-sent' != get_post_meta( $this->order->get_id(), '_mf_receipt_status', true ) ) {
			return false;
		}

		$customer = $this->get_customer();
		if (!empty($customer)) {
			if (empty($customer['CustomerNumber'])) {
				$customer['CustomerNumber'] = $this->get_customer_number($customer);
				if ( ! $customer['CustomerNumber'] ) {
					$customer['CustomerNumber'] = null;
				}
			}
		}

		// if ( ! isset( $customer['CustomerNumber'] ) || ! $customer['CustomerNumber'] ) {
		// 	return false;
		// }
		
		$account_codes = array_map( function ( $mf_account_code ) {
			return \MF_Sync_Receipts_Factory::create_account_code([
				'id' => $mf_account_code->AccountCode->Id,
				'account_code' => $mf_account_code->AccountCode->Account,
			]);
		}, $mf_account_codes );
		$company_corp_no = $api_options['mf_company_corp_no'];
		$company_name = $api_options['mf_company_name'];
		$customer_number = $customer['CustomerNumber'];
		$shipping_sku = $api_options['mf_shipping_product_id'];
		$payment_method_accounts = $api_options['bookkeeping_account_payments'];
		$shipping_account = $api_options['mf_bookkeeping_account_shipping'];
		$tax_account_0 = \MF_Sync_Bookkeeping::get_vat_account( 0 );
		$tax_account_6 = \MF_Sync_Bookkeeping::get_vat_account( 6 );
		$tax_account_12 = \MF_Sync_Bookkeeping::get_vat_account( 12 );
		$tax_account_25 = \MF_Sync_Bookkeeping::get_vat_account( 25 );
		$custom_tax_account = $api_options['mf_bookkeeping_account_tax'];
		if ( $custom_tax_account ) {
			$tax_account_0 = $tax_account_0 ? $tax_account_0 : $custom_tax_account;
			$tax_account_6 = $tax_account_6 ? $tax_account_6 : $custom_tax_account;
			$tax_account_12 = $tax_account_12 ? $tax_account_12 : $custom_tax_account;
			$tax_account_25 = $tax_account_25 ? $tax_account_25 : $custom_tax_account;
		}

		// Check if there are any quantities, else set restock to false
        $order_items = $order_refund->get_items();
		if ( empty( $order_items ) ) {
			$restock_items = false;
		} else {
			$quantity_order_items = array_filter( $order_items, function ( $order_item ) {
				return $order_item->get_quantity() != 0;
			} );
			if ( count( $quantity_order_items ) < 1 ) {
				$restock_items = false;
			}
		}

		return \MF_Sync_Receipts_Factory::create_receipt(
			$order_refund,
			$account_codes,
			$company_corp_no,
			$company_name,
			$customer_number,
			$shipping_sku,
			$payment_method_accounts,
			$shipping_account,
			$tax_account_0,
			$tax_account_6,
			$tax_account_12,
			$tax_account_25,
			$restock_items
		);
	}

	public function order_to_moreflo_receipt() {
		if ( ! moreflo_sync_receipts_enabled() ) {
			return false;
		}

		if ( ! $this->needs_receipt() ) {
			return false;
		}

		$customer = $this->get_customer();
		if (!empty($customer)) {
			if (empty($customer['CustomerNumber'])) {
				$customer['CustomerNumber'] = $this->get_customer_number($customer);
				if ( ! $customer['CustomerNumber'] ) {
					$customer['CustomerNumber'] = null;
				}
			}
		}

		// if ( ! isset( $customer['CustomerNumber'] ) || ! $customer['CustomerNumber'] ) {
		// 	return new \WP_Error( 'missing_customer_number', __( 'Customer Number missing', 'wpmf' ) );
		// }

		$api_options = get_option('moreflo_api_url');
		$mf_account_codes = get_option('mf_account_ids');
		
		$account_codes = array_map( function ( $mf_account_code ) {
			return \MF_Sync_Receipts_Factory::create_account_code([
				'id' => $mf_account_code->AccountCode->Id,
				'account_code' => $mf_account_code->AccountCode->Account,
			]);
		}, $mf_account_codes );
		$company_corp_no = $api_options['mf_company_corp_no'];
		$company_name = $api_options['mf_company_name'];
		$customer_number = $customer['CustomerNumber'];
		$shipping_sku = $api_options['mf_shipping_product_id'];
		$payment_method_accounts = $api_options['bookkeeping_account_payments'];
		$shipping_account = $api_options['mf_bookkeeping_account_shipping'];
		$tax_account_0 = \MF_Sync_Bookkeeping::get_vat_account( 0 );
		$tax_account_6 = \MF_Sync_Bookkeeping::get_vat_account( 6 );
		$tax_account_12 = \MF_Sync_Bookkeeping::get_vat_account( 12 );
		$tax_account_25 = \MF_Sync_Bookkeeping::get_vat_account( 25 );
		$custom_tax_account = $api_options['mf_bookkeeping_account_tax'];
		if ( $custom_tax_account ) {
			$tax_account_0 = $tax_account_0 ? $tax_account_0 : $custom_tax_account;
			$tax_account_6 = $tax_account_6 ? $tax_account_6 : $custom_tax_account;
			$tax_account_12 = $tax_account_12 ? $tax_account_12 : $custom_tax_account;
			$tax_account_25 = $tax_account_25 ? $tax_account_25 : $custom_tax_account;
		}
		$do_stock_management = false;
		if ( isset( $api_options['sync_new_orders_to_apps'] ) && 'disabled' == $api_options['sync_new_orders_to_apps'] ) {
			$do_stock_management = true;
		}

		return \MF_Sync_Receipts_Factory::create_receipt(
			$this->order,
			$account_codes,
			$company_corp_no,
			$company_name,
			$customer_number,
			$shipping_sku,
			$payment_method_accounts,
			$shipping_account,
			$tax_account_0,
			$tax_account_6,
			$tax_account_12,
			$tax_account_25,
			$do_stock_management
		);
	}

    private function needs_receipt() {
		$api_options = get_option(MOREFLO_API_URL);

		// Is receipts enabled?
		if ( ! isset( $api_options['is_receipt_active'] ) || 'enabled' != $api_options['is_receipt_active'] ) {
			return false;
		}

		// Company corp no, name and bookable account set?
		if ( ! $api_options['mf_company_corp_no'] || ! $api_options['mf_company_name'] ) {
			return false;
		}

		// Only create for orders that needs receipts
		if ( 'needs-receipt' != get_post_meta( $this->order->get_id(), '_mf_receipt_status', true ) ) {
			return false;
		}

		// Only create receipts on payed orders
		if ( is_null( $this->order->get_date_paid() ) ) {
			return false;
		}

        return true;
    }

	private function get_customer_number($customer)
	{
		$MfCusId = false;
		$wooCusID = false;
		if(get_user_by('email', $customer['Email']))
		{
			$woocustomer = get_user_by('email', $customer['Email']);
			$wooCusID = $woocustomer->ID;
			$MfCusId = get_user_meta($wooCusID, 'mf_customer_id', true);
		}
		if(!empty($MfCusId))
		{
			MfLog::log(DEBUG, 'Skipping get_customer API call. mf_customer_id#'.$MfCusId.' existing on woo side for customer#'.$wooCusID);
			return $MfCusId;//If customer Id already recorded in user meta return else run on api to search customer
		}

		$query = [
			"SearchPhrase"=> $customer['Email'],
			"SearchType"=> "Email",
			"PageSize"=> 0,
			"ExactMatch"=> true
		];

		$api = new \MfApi();
		$customerNumbers = $api->get_customer($query);

		if(!empty($customerNumbers['list']) && is_array($customerNumbers['list']))
		{
			$MfCusId = $customerNumbers['list'][0]['Customer']['CustomerNumber'];//Set first result from results as customer number
			if(!empty($wooCusID))
			{
				//If user use guest account and user email already registered on Woo site update usermeta
				//Or user is already registered user dave mf_customer_id on user meta
				update_user_meta($wooCusID, 'mf_customer_id', $MfCusId );
				MfLog::log(DEBUG, 'Upodate User#'.$wooCusID.' with mf_customer_id#'.$MfCusId);
			}
		}
		MfLog::log(DEBUG, 'return customer number #'.$MfCusId.'#');
		return $MfCusId;
	}

	private function get_order_pay_method() {
		if($this->getPayMethod() == 'cod' || $this->getPayMethod() == 'bacs')
		{
			return 'Cashier';
		} elseif ( $this->getPayMethod() == 'moreflo_invoice' ) {
			return 'Invoice';
		} else {
			return 'Web';
		}
	}

	private function getPayMethod() {
		return $this->order->get_payment_method();
	}

	private function get_order_pay_status() {
		 // If the order has been fully paid, it should not revert when set to on-hold (it needs to be credited)
		$order_has_been_fully_paid = get_post_meta( $this->order->get_id(), 'mf_order_has_been_fully_paid', true );
		if ( $order_has_been_fully_paid ) {
			return 'FullyPaid';
		}

		$order_status = $this->order->get_status();
		$mf_pay_status = 'NotSet';
		switch ($order_status) {
			case 'cancelled':
			case 'refunded':
			case 'failed':
				$mf_pay_status = 'NotSet';
				break;			
			case 'pending':
			case 'on-hold':
				$mf_pay_status = 'NotPaid';
				break;
			case 'processing':
				if ($this->getPayMethod() == 'cod'){//This is to avoid cash on delivery processing not to be change as fully paid
					$mf_pay_status = 'NotPaid';
				} elseif ( $this->getPayMethod() == 'moreflo_invoice' ) {
					$mf_pay_status = 'NotPaid';
				} else {
					$mf_pay_status = 'FullyPaid';
					update_post_meta( $this->order->get_id(), 'mf_order_has_been_fully_paid', true );
				}
				break;						
			case 'completed':
				$mf_pay_status = 'FullyPaid';
				update_post_meta( $this->order->get_id(), 'mf_order_has_been_fully_paid', true );
				break;
			default:
			break;
		}
		return $mf_pay_status;
	}

	private function get_order_status($order_type) {

		$order_status = $this->order->get_status();
		$appointStatus = $this->get_appointment_status();
		$mf_status = 'Created';

		switch ($order_status) {
			case 'pending':
				$mf_status = 'Created';
				break;				
			case 'on-hold':
				$mf_status = 'Started';
				break;
			case 'processing':
			case 'completed':
				$mf_status = 'Closed';
				break;
			case 'cancelled':
			case 'refunded':
			case 'failed':
			case 'kco_incomplete':
				$mf_status = 'Cancelled';
				break;
			default:
			break;
		}

		$payment_completes_status = ['completed', 'processing'];
		if(in_array($order_status,$payment_completes_status) && empty($this->order->get_transaction_id()))
		{
			$mf_status = 'ReadyForHandOut';
		}
		if($order_type == 'SERVICE_ORDER'){
			if(in_array($order_status,$payment_completes_status) )
			{
				$mf_status = 'ReadyForHandOut';
			}
			if(
				($order_status == 'pending' || $order_status == 'on-hold') 
				&& $appointStatus == 'confirmed' 
				&& ($this->getPayMethod() == 'cod' || $this->getPayMethod() == 'bacs' || $this->getPayMethod() == 'moreflo_invoice')
				)
			{
				$mf_status = 'ReadyForHandOut';
			}
		}
		//Order v3 integration
		if($order_type == 'NORMAL_ORDER' && in_array($order_status,$payment_completes_status) && $this->getPayMethod() != 'cod' && $this->getPayMethod() != 'moreflo_invoice' ){
			if($this->getLocalPickupshipping())
			{
				$mf_status = 'ReadyForHandOut';
			}
			else
			{
				$mf_status = 'Delivered';
			}
		}

		return $mf_status;
	}

	private function get_customer() {

		$mfCusId = get_post_meta($this->order->get_id(), 'mf_customer_id', true);

		$customer = array(
			'CustomerNumber'	=> (isset($mfCusId))? $mfCusId : '' ,
			'Name'              => trim($this->order->get_billing_first_name() . ' ' . $this->order->get_billing_last_name()),
			'Address1'          => $this->order->get_billing_address_1(),
			'Address2'          => $this->order->get_billing_address_2(),
			'PostCode'          => $this->order->get_billing_postcode(),
			'City'              => $this->order->get_billing_city(),
			'Country'           => $this->order->get_billing_country(),
			'Email'             => $this->order->get_billing_email(),
			'Mobile'        => $this->order->get_billing_phone(),
		);

		foreach ($customer as $key => $v) {
			if(empty($v) && !in_array($key, array('CustomerNumber')) ) {
				unset($customer[$key]);
			}
		}

		return (isset($customer['Name']) ? $customer : null);
	}

	private function get_billing_address() {

		$address = array(
			'Name'              => trim($this->order->get_billing_first_name() . ' ' . $this->order->get_billing_last_name()),
			'Address1'          => $this->order->get_billing_address_1(),
			'Address2'          => $this->order->get_billing_address_2(),
			'PostCode'          => $this->order->get_billing_postcode(),
			'City'              => $this->order->get_billing_city(),
			'Country'           => $this->order->get_billing_country(),
			'Email'             => $this->order->get_billing_email(),
			'Mobile'        => $this->order->get_billing_phone()
		);

		foreach ($address as $key => $v) {
			if(empty($v)) {
				unset($address[$key]);
			}
		}

		return (isset($address['Name']) ? $address : null);
	}

	private function get_shipping_address() {

		$address = array(
			'Name'              => trim($this->order->get_shipping_first_name() . ' ' . $this->order->get_billing_last_name()),
			'Address1'          => $this->order->get_shipping_address_1(),
			'Address2'          => $this->order->get_shipping_address_2(),
			'PostCode'          => $this->order->get_shipping_postcode(),
			'City'              => $this->order->get_shipping_city(),
			'Country'           => $this->order->get_shipping_country()
		);

		foreach ($address as $key => $v) {
			if(empty($v)) {
				unset($address[$key]);
			}
		}

		return (isset($address['Name']) ? $address : null);
	}

	private function get_order_row_tax($product) {
		$tax_percentage = 0;
		$wc_tax = new WC_Tax();

		$billing_country = $this->order->get_billing_country();

		$tax_class = $product->get_tax_class();

		$tax_data = $wc_tax->get_rates( $tax_class );
		
		foreach ($tax_data as $tax_rate) {
			$tax_percentage = $tax_rate['rate'];
		}

		return $tax_percentage;
	}

	private function get_order_rows() {

		$order_rows = array();
		$order_items = $this->order->get_items();

		foreach ($order_items as $item_data) {

			// Uses receipt calculations to avoid duplicate code
			$receipt_article = MF_Sync_Receipts_Factory::create_article( $item_data, $this->order, 0, [], '' );

			if ( is_null( $receipt_article ) ) {
				// Not a valid row
				continue;
			}

			$product = $item_data->get_product();
			$product_type = WC_Product_Factory::get_product_type($product->get_id());
			$this->product_types[] = $product_type;

			$item = array(
				ARTICLENUMBER => $receipt_article->get_sku(),
				ARTICLENAME => $receipt_article->get_name(),
				QUANTITY => $receipt_article->get_quantity(),
				'Discount' => mf_wc_round(
					$receipt_article->get_total_discount(),
					wc_get_rounding_precision()
				),
			);

			if(in_array($product_type, array(BOOKING, APPOINTMENT))) {
				$item[RETAILPRICE] = $receipt_article->get_unit_price_without_discounts();
				$item['Vat'] = $receipt_article->get_vat_rate();
			} else {
				$item['UnitAmountInclVatExclDiscount'] = $receipt_article->get_unit_price_without_discounts();
				$item['VatRate'] = $receipt_article->get_vat_rate();
			}

			if(!empty($item[ARTICLENUMBER]) && !empty($item[QUANTITY])) {
				$order_rows[] = $item;

				$order_rows = apply_filters( 'moreflo_sync_add_order_row_after', $order_rows, $item_data, $product );
			}
		}

		$shipping_row = $this->get_shipping_order_row();

		if(is_array($shipping_row)) {
			$order_rows[] = $shipping_row;
		}


		return $order_rows;
	}

	private function get_refund_order_rows( $parent_order_id ) {
		$order_rows = array();
		$order_items = $this->order->get_items( [ 'line_item', 'shipping' ] );
		$parent_order = wc_get_order( $parent_order_id );

		$settings = get_option( 'moreflo_api_url' );
		$refund_part_sku = isset( $settings['mf_part_refund_product_id'] )
			? $settings['mf_part_refund_product_id']
			: '';
		$shipping_vat = 25.0; // Default value, is updated below
		$shipping_product_sku = ( isset( $settings['mf_shipping_product_id'] ) )
			? $settings['mf_shipping_product_id']
			: '';

		if ( $shipping_product_sku ) {
			$shipping_product_id = wc_get_product_id_by_sku( $shipping_product_sku );
			$shipping_product = wc_get_product( $shipping_product_id );

			if ( is_a( $shipping_product, 'WC_Product' ) ) {
				$shipping_vat = $this->get_order_row_tax( $shipping_product );
			}
		}

		foreach ( $order_items as $item_data ) {
			if ( is_a( $item_data, 'WC_Order_Item_Shipping' ) && $shipping_product_sku ) {
				$product_type = 'shipping';
				$price = abs( $item_data->get_total() + $item_data->get_total_tax() );
				$vat = $shipping_vat;
				$product_sku = $shipping_product_sku;
				$product_name = __( 'Shipping', 'moreflo-sync' );
				$quantity = -1;
				$dis = 0;
			} elseif ( is_a( $item_data, 'WC_Order_Item_Product' ) ) {
				$product = $item_data->get_product();
				$product_sku = $product->get_sku();
				$quantity = $item_data->get_quantity();

				$product_type = WC_Product_Factory::get_product_type( $product->get_id() );

				$price = $product->get_regular_price();
				$vat = $this->get_order_row_tax( $product );
				$product_name = $product->get_name();
				$dis = 0;

				$this->product_types[] = $product_type;

				if ( $quantity == 0 ) {
					if ( ! $refund_part_sku ) {
						// Refund part sku has not been set, cannot create row for this product
						continue;
					}

					$quantity = -1;
					$product_sku = $refund_part_sku;
					$price = abs( $item_data->get_total() + $item_data->get_total_tax() );
				} else {
					$original_salesprice = $parent_order->get_item_subtotal( $item_data, true, false );
					
					if ( $original_salesprice < $price ) {
						$dis = $price * $quantity - $original_salesprice * $quantity;
					}
				}
			} else {
				continue;
			}


			$item = array(
				ARTICLENUMBER => $product_sku,
				ARTICLENAME => $product_name,
				QUANTITY => $quantity,
			);

			if ( in_array( $product_type, array( BOOKING, APPOINTMENT ) ) ) {
				// Not supported
				continue;
			} else {
				$item['UnitAmountInclVatExclDiscount'] = $price;
				$item['VatRate'] = $vat;
				$item['Discount'] = $dis;
			}

			if ( ! empty( $item[ARTICLENUMBER] ) && ! empty( $item[QUANTITY] ) ) {
				$order_rows[] = $item;
			}
		}

		return $order_rows;
	}

	function get_shipping_order_row() {
		$mf_settings = get_option(MOREFLO_API_URL);
		$mf_shipping_product_id = '';

		if(!empty($mf_settings['mf_shipping_product_id'])) {
			$mf_shipping_product_id = $mf_settings['mf_shipping_product_id'];
		}

		if($mf_shipping_product_id) {
			$shipping_tax_rate = 0;
			$shipping_tax = $this->order->get_shipping_tax();
			$shipping_total = floatval(round( $this->order->get_shipping_total() + $shipping_tax, wc_get_price_decimals() ));

			if($shipping_tax > 0) {
				$tax_lines = $this->order->get_items( 'shipping' );
				if ( ! empty( $tax_lines ) ) {
					foreach ( $tax_lines as $tl ) {
						$shipping_tax_total = array_shift( $tl->get_taxes()['total'] );
						$shipping_tax_rate = floatval( ceil( 100 * ( ( ( floatval( $this->order->get_shipping_total() ) + floatval( $shipping_tax_total ) ) / floatval( $this->order->get_shipping_total() ) ) - 1 ) ) );
					}
				}
			}

			if($shipping_total > 0) {
				return array(
					ARTICLENUMBER                 => $mf_shipping_product_id,
					ARTICLENAME                   => __('Shipping', 'wpmf'),
					'UnitAmountInclVatExclDiscount' => $shipping_total,
					//'Discount'                      => 0,
					'VatRate'                       => $shipping_tax_rate,
					QUANTITY                      => 1,
				);
			}
		}

		return false;
	}

	function validate_moreflo_order_response ($type, $action, $response) {

		$order_id = $this->order->get_id();

		if($response) {

			$order = json_decode($response);
			$order_data = get_post_meta($order_id,'mf_order', true);
			$mf_cus_Id;

			if(empty($order_data[MF_ORDER_ID]) && $type == NORMAL && !empty($order->list[0]->Success) && !empty($order->list[0]->Order->Number)) {

				$order_data = array(
					'mf_order_no' => $order->list[0]->Order->Number,
					MF_ORDER_ID => $order->list[0]->Order->Id,
					MF_ORDER_TYPE => $type,
				);
				update_post_meta($order_id,'mf_order', $order_data);
				update_post_meta($order_id,'mf_id', $order_data[MF_ORDER_ID]);

			} else if(empty($order_data[MF_ORDER_ID]) && $type == SERVICE && !empty($order->list[0]->Success) && !empty($order->list[0]->ServiceOrder->Number)) {
				$order_data = array(
					'mf_order_no' => $order->list[0]->ServiceOrder->Number,
					MF_ORDER_ID => $order->list[0]->ServiceOrder->Id,
					MF_ORDER_TYPE => $type,
				);
				$mf_cus_Id = $order->list[0]->ServiceOrder->Customer->CustomerNumber;
				update_post_meta($order_id,'mf_order', $order_data);
				update_post_meta($order_id,'mf_id', $order_data[MF_ORDER_ID]);
			}
			if(!empty($mf_cus_Id))
			{
				update_post_meta($order_id,'mf_customer_id', $mf_cus_Id);
			}

			if(!empty($order->list[0]->Success)) {
				if($action == 'update') {
					$this->add_order_note(__(
						sprintf('Moreflo %s order(#%s) updated successfully.', $type, $order_data['mf_order_no']),
						'wpmf'
					));
				} else {
					$this->add_order_note(__(
						sprintf('Moreflo %s order(#%s) added successfully.', $type, $order_data['mf_order_no']),
						'wpmf'
					));
				}

				return true;
			}

			$this->add_order_note($response);
		}

		$this->add_order_note(__('Moreflo order sync failed. Please retry again by saving the order.', 'wpmf'));
		return false;

	}
	public function updateArticleQunatity($articleNo)
	{
		$api = new \MfApi();
		$article_instance = new \MfArticles();
		$response = $api->get_article($articleNo);
		$article = json_decode($response, true);
		if(function_exists('wc_get_product_id_by_sku') && $article['Article'][ISSTOCKITEM] == true){
			$product_id = wc_get_product_id_by_sku($article[ARTICLENUMBER]);
			$article_instance->update_stock($product_id, $article['Article'][QUANTITY]);
			MfLog::log(DEBUG, 'Reset stock for product ID#'.$product_id. ', Quantity:'.$article['Article'][QUANTITY].' Due to order status#'.$this->order->get_status());
			$this->add_order_note(__('Reset stock for product ID#'.$product_id. ', Quantity:'.$article['Article'][QUANTITY].' Due to order status#'.$this->order->get_status(), 'wpmf'));
		}
	}
	
	public static function transform_mf_order($response) {

		$order = null;

		if($response) {
			$mf_order = json_decode($response);

			if(!empty($mf_order->ServiceOrder->Id)) {
				$order = $mf_order->ServiceOrder;
			} else if (!empty($mf_order->Order->Id)) {
				$order = $mf_order->Order;
			}
		}

		return $order;

	}

	function process_stocks() {

		$order_rows = array();

		foreach ($this->order->get_items() as $item_data) {

			$product = $item_data->get_product();

			if($product instanceof WC_Product) {

				if($product->is_virtual()) {
					MfLog::log(DEBUG, 'Stock update for the following item has failed due to it\'s a virtual product');
					MfLog::log(DEBUG, print_r($item_data, true));
					continue;
				}

				$item = array(
					ARTICLENUMBER                 => $product->get_sku(),
					QUANTITY                      => abs($item_data->get_quantity())
				);

				if(!empty($item[ARTICLENUMBER]) && !empty($item[QUANTITY])) {
					$order_rows[] = $item;
				} else {
					MfLog::log(DEBUG, 'Stock update for the following item has failed due to no article no or quantity');
					MfLog::log(DEBUG, print_r($item_data, true));
				}

			} else {
				MfLog::log(DEBUG, 'Stock update for the following item has failed due to it\'s not a WC_Product');
				MfLog::log(DEBUG, print_r($item_data, true));
			}
		}

		return $order_rows;

	}

	function get_order_type() {

		$PTsValueCount = array_count_values($this->product_types);
		if((isset($PTsValueCount[APPOINTMENT]) && $PTsValueCount[APPOINTMENT] >= 2) || (isset($PTsValueCount[APPOINTMENT]) && $PTsValueCount[BOOKING] >= 2)) {
			return 'INVALID_ORDER';	
		}
		
		if(in_array(BOOKING, $this->product_types) || in_array(APPOINTMENT, $this->product_types)) {
			return 'SERVICE_ORDER';
		}

		if(!in_array(BOOKING, $this->product_types) && !in_array(APPOINTMENT, $this->product_types)) {
			return 'NORMAL_ORDER';
		}
		return 'INVALID_ORDER';
	}

	private function get_appointment_booking() {

		if(function_exists('wc_appointments_maybe_appointment_object')) {
			$booking = wc_appointments_maybe_appointment_object($this->order->get_order_number());

			if($booking && $booking instanceof \WC_Appointment) {
				$start_date = $booking->get_start_date('c', '');
				$staff_id = $booking->get_staff_ids();

				return array(
					'BookingStartTime' => $start_date,
					'BookingEndTime' => $booking->get_end_date('c', ''),
					'StartDate' => $start_date,
					'staff_id' => $staff_id,
				);
			}

		}

		return false;
	}

	private function get_wpbooking_booking() {

		if ( function_exists( 'get_wc_booking') ) {
			$booking_data = new \WC_Booking_Data_Store();
			$booking_ids = $booking_data->get_booking_ids_from_order_id( $this->order->get_id() );

			if(!empty($booking_ids[0])) {
				$booking = new \WC_Booking( $booking_ids[0] );

				if($booking instanceof \WC_Booking) {

					$start_date = $booking->get_start_date('c', '');

					return array(
						'BookingStartTime' => $start_date,
						'BookingEndTime' => $booking->get_end_date('c', ''),
						'StartDate' => $start_date,
					);
				}
			}
		}

		return false;
	}

	private function get_booking_data() {

		$wp_appointment = $this->get_appointment_booking();

		if($wp_appointment) {
			return $wp_appointment;
		}

		return $this->get_wpbooking_booking();
	}

	public function add_order_note($note) {

		if($this->order && $this->order instanceof WC_Order) {
			$this->order->add_order_note($note);
		}

		MfLog::log(DEBUG, sprintf('[Order Note] %s', $note));
	}

	public static function get_all() {
		$query = new WC_Order_Query(array(
			'status' => array(
				'pending',
				'processing',
				'on-hold'
			),
			'limit' => -1,
		));

		return $query->get_orders();
	}

	// public function update_order($mf_status) {
	public function update_order($mf_status, $mf_pay_status, $mf_order_type) {

		if($this->order) {

			$mf_status = trim(strtolower($mf_status));
			$order_id = $this->order->get_id();
			$current_wc_status = $this->order->get_status();
			$current_appointment_status = $this->get_appointment_status();

			$new_status = $this->get_wc_order_status_from_mf_status($mf_status, $mf_order_type);//Service and booking for appointment status, normal order for woo orders
			$mf_payment_status = $this->get_wc_order_status_from_mf_pay_status($mf_pay_status);	
			$existing_mf_status_in_wc = $this->get_existing_mf_booking_order_status_from_wc_status();		

			if($mf_order_type == SERVICE )
			{
				$existing_mf_status_in_appointment = $this->get_existing_mf_booking_order_status_from_appointment_status();
				$valid_mf_status = array('new', 'waitingconfirmation', 'confirmed', 'cancelled', 'created', 'started', 'readyforhandout', 'closed');

				if(in_array($mf_status, $valid_mf_status) &&  $mf_status != $existing_mf_status_in_appointment)
				{

					if(!empty($new_status) && $new_status != $current_appointment_status)
					{ 
						$appointment = $this->getAppoinmentObj();
						$shared_status = ['confirmed', 'paid'];
						if($new_status == 'confirmed' && in_array($current_appointment_status, $shared_status))
						{
							MfLog::log(DEBUG, sprintf('Order(%s) not updated appointment status. Becuase received status as confirmed and it has shared status', $order_id));
							return;
						}
						if($appointment)
						{
							$props = array(
								'status' => $new_status,
							);
							$appointment->set_props( $props );
							$appointmentID = $appointment->save();
				
							if($appointmentID)
							{
								MfLog::log(DEBUG, sprintf('Appointment(%s) status updated', $appointmentID));
								if($mf_status == 'cancelled')
								{
									$this->order->update_status('cancelled', __('Order status synced with Moreflo', 'wpmf'));
									return;
								}
							}
							else
							{
								MfLog::log(DEBUG, sprintf('Failed to update Order(%s) appointments status', $order_id));
							}
						}

					}
					else
					{
						MfLog::log(DEBUG, sprintf('Order(%s) update failed in Appointment. (No mapped MF status/existing WC status)', $order_id));
						MfLog::log(DEBUG, sprintf('Current Status: %s / New Status: %s', $current_appointment_status, $new_status));
					}
				}
				else
				{
					MfLog::log(DEBUG, sprintf('Order(%s) update failed in Appointment. (Not a valid MF status/existing MF status)', $order_id));
					MfLog::log(DEBUG, sprintf('Current Status: %s / New Status: %s', $existing_mf_status_in_appointment, $mf_status));
				}

				if($existing_mf_status_in_wc != $mf_pay_status)
				{					
					if($mf_pay_status == 'FullyPaid' && empty($this->order->get_transaction_id())){
						update_post_meta( $order_id, '_payment_method',  'PayedFromCashier');
						update_post_meta( $order_id, '_payment_method_title', __('Payed From Cashier', 'wpmf'));
						update_post_meta( $order_id, '_transaction_id', time());
						$this->order->payment_complete();
					}else{
						$this->order->update_status($mf_payment_status, __('Order status synced with Moreflo', 'wpmf'));
					}
					$mf_order = $this->order_to_moreflo_order();
					$new_hash = generate_order_hash($mf_order);
					update_post_meta($order_id, MF_HASH, $new_hash);				
				}
				else
				{
					MfLog::log(DEBUG, sprintf('Order(%s) update failed. (Not a valid MF status/existing MF status)', $order_id));
					MfLog::log(DEBUG, sprintf('Current Status: %s / New Status: %s', $existing_mf_status_in_wc, $mf_pay_status));
				}
			}
			else
			{
				if($existing_mf_status_in_wc != $mf_pay_status)
				{

					if($mf_pay_status == 'FullyPaid' && empty($this->order->get_transaction_id())){
						update_post_meta( $order_id, '_payment_method',  'PayedFromCashier');
						update_post_meta( $order_id, '_payment_method_title', __('Payed From Cashier', 'wpmf'));
						update_post_meta( $order_id, '_transaction_id', time());
						update_post_meta( $order_id, 'mf_order_has_been_fully_paid', true);
						$this->order->payment_complete();
					}

					$mf_order = $this->order_to_moreflo_order();
					$new_hash = generate_order_hash($mf_order);
					update_post_meta($order_id, MF_HASH, $new_hash);
				} else {
					MfLog::log(DEBUG, sprintf('Order(%s) update failed in WC. (No mapped MF status/existing WC status)', $order_id));
					MfLog::log(DEBUG, sprintf('Current Status: %s / New Status: %s', $existing_mf_status_in_wc, $mf_pay_status));
				}
			}
		}

	}

	private function get_existing_mf_booking_order_status_from_wc_status() {
		$current_status = $this->order->get_status();
		$new_mf_status = '';

		switch ($current_status) {
			case 'cancelled':
			case 'refunded':
			case 'failed':
				$new_mf_status = 'NotSet';
				break;			
			case 'pending':				
			case 'on-hold':
				$new_mf_status = 'NotPaid';
				break;
			case 'processing':				
			case 'completed':
				$new_mf_status = 'FullyPaid';
				break;
			default:
			break;
		}
		return $new_mf_status;
	}

	private function get_existing_mf_booking_order_status_from_appointment_status() {
		$appointStatus = $this->get_appointment_status();
		switch ($appointStatus) {
			case 'unpaid':
				$mf_status = 'new';
				break;					
			case 'pending-confirmation':
				$mf_status = 'waitingconfirmation';
				break;
			case 'confirmed':
				$mf_status = 'confirmed';
				break;
			case 'paid':
				$mf_status = 'confirmed';
				break;
			case 'complete':
				$mf_status = 'completed';
				break;
			case 'cancelled':
				$mf_status = 'cancelled';
				break;
			default:
			break;
		}
	}

	private function get_appointment_status()
	{
		$appointment = $this->getAppoinmentObj();
			if($appointment){
				$appointment_status = $appointment->get_status();
				return esc_attr($appointment_status);
			}
	}

	// private function get_existing_mf_order_status_from_wc_status() {

	// 	$current_status = $this->order->get_status();
	// 	$local_pickup = $this->getLocalPickupshipping();//New changes 20-05-2020
	// 	$new_mf_status = '';

	// 	if($current_status == 'pending' || $current_status == 'on-hold') {
	// 		$new_mf_status = 'created';
	// 	}
	// 	else if($current_status == 'processing') {
	// 		$new_mf_status = 'started';
	// 	}
	// 	else if($current_status == 'cancelled' || $current_status == 'refunded') {
	// 		$new_mf_status = 'cancelled';
	// 	}
	// 	else if($current_status == 'completed' && !empty($this->order->get_transaction_id())) {
	// 		$new_mf_status = 'closed';
	// 	}
	// 	//else if($current_status == 'completed') {//New changes 17-03-2020
	// 	else if($current_status == 'completed' && empty($local_pickup)) {//New changes 20-05-2020
	// 		$new_mf_status = 'PayedFromExternal';
	// 	}
	// 	else if($current_status == 'completed') {
	// 		$new_mf_status = 'readyforhandout';
	// 	}

	// 	return $new_mf_status;
	// }

	private function get_wc_order_status_from_mf_status($mf_status, $mf_order_type) {

		$new_status = '';

		if($mf_order_type == NORMAL){
			switch ($mf_status) {
				case 'created':
					$new_status = 'pending';
					break;
				case 'started':
					$new_status = 'on-hold';
					break;
				case 'cancelled':
					$new_status = 'cancelled';
					break;
				case 'delivered':
					$new_status = 'processing';
					break;			
				case 'readyforhandout':
					$new_status = 'processing';
					break;
				default:
				break;
			}
		}else{
            switch ($mf_status) {//Below is mapping for appoinment status
                case 'created':
                    $new_status = 'unpaid';
                    break;
                case 'started':
                    $new_status = 'pending-confirmation';
                    break;
                case 'readyforhandout':
                    $new_status = 'confirmed';
                    break;
                case 'closed':
                    $new_status = 'complete';
                    break;					
                case 'cancelled':
                    $new_status = 'cancelled';
                    break;									
                default:
                break;
            }
		}
		return $new_status;
	}

	private function get_wc_order_status_from_mf_pay_status($mf_pay_status) {
		$new_status = '';
		switch ($mf_pay_status) {
			case 'FullyPaid':
				$new_status = 'processing';
				break;
			case 'NotPaid':
				$new_status = 'on-hold';
				break;
			case 'NotSet':
				$new_status = 'cancelled';
				break;					
			default:
			break;
		}
		return $new_status;
	}

	public function getLocalPickupshipping()
	{
		$shipping_methods = $this->order->get_shipping_methods();
		$localpickup = false;

		foreach( $shipping_methods as $item ){
			$shipping_method_id = $item->get_method_id();
			if ( $shipping_method_id === 'local_pickup') {
				$localpickup = true;
			}
		}

		return $localpickup;
	}

	public function UpdateTimeSlots($bookingData)
	{
		MfLog::log(DEBUG, 'Updating appointment times startes');
		$appointment = $this->getAppoinmentObj();
		$appointmentID;

		if($appointment)
		{
			$start_date		= explode( '-', $bookingData['StartDate']);
			$end_date		= explode( '-', $bookingData['StopDate']);
			$start_time		= explode( ':', $bookingData['StartTime']);
			$end_time		= explode( ':', $bookingData['StopTime']);
			$start			= mktime( $start_time[0], $start_time[1], 0, $start_date[1], $start_date[2], $start_date[0] );
			$end			= mktime( $end_time[0], $end_time[1], 0, $end_date[1], $end_date[2], $end_date[0] );

			$props = array(
				'start'           => $start,
				'end'             => $end,
			);
			$appointment->set_props( $props );
			$appointmentID = $appointment->save();

			if($appointmentID)
			{
				MfLog::log(DEBUG, 'Order number : '.$this->order->get_order_number().' start and end date updated with appoinment number '.$appointmentID);
			}
			else
			{
				MfLog::log(DEBUG, 'Error saving appointments dates');
			}
			return $appointmentID;
		}
	}

	private function getAppoinmentObj()
	{
		if(function_exists('wc_appointments_maybe_appointment_object'))
		{
			$appointment = wc_appointments_maybe_appointment_object($this->order->get_order_number());
			if($appointment && $appointment instanceof \WC_Appointment)
			{
				return $appointment;
			}

		}

		return false;
	}

	public function orderTotal()
	{
		return $this->order->get_total();
	}
}

