<?php

defined( 'ABSPATH' ) || exit;

$pre_api_options = get_option(MOREFLO_API_URL);

if ( $pre_api_options ) {
    $pre_active_status = $pre_api_options['is_active'];
    $pre_test_url = $pre_api_options['api_test_url'];
    $pre_test_user = $pre_api_options['api_test_user'];
    $pre_test_pw = $pre_api_options['api_test_pw'];
    $pre_production_url = $pre_api_options['api_production_url'];
    $pre_production_user = $pre_api_options['api_production_user'];
    $pre_production_pw = $pre_api_options['api_production_pw'];
    $mf_sync_articles_interval = $pre_api_options[MF_SYNC_ARTICLES_INTERVAL];
    $pre_active_price_status = $pre_api_options[ISPRICEACTIVE];
    $pre_active_pic_status = $pre_api_options['is_pic_active'];
    $sync_article_description = ( isset( $pre_api_options['sync_article_description'] ) && 'enabled' == $pre_api_options['sync_article_description'] )
        ? 'enabled'
        : 'disabled';
    $pre_active_product_title = $pre_api_options['is_product_title_active'];            

    $sync_new_orders_to_apps = ( isset( $pre_api_options['sync_new_orders_to_apps'] ) )
        ? $pre_api_options['sync_new_orders_to_apps']
        : 'enabled';
    $sync_publish_products_status = ( isset( $pre_api_options['sync_publish_products_status'] ) )
        ? $pre_api_options['sync_publish_products_status']
        : 'disabled';

	$sync_only_product_stock_amount = ( isset( $pre_api_options['sync_only_product_stock_amount'] ) )
        ? $pre_api_options['sync_only_product_stock_amount']
        : 'disabled';

    $mf_sync_status = isset( $pre_api_options[MF_SYNC_STATUS] )
        ? $pre_api_options[MF_SYNC_STATUS]
        : 'disable';
    $mf_sync_items_per_page = isset( $pre_api_options['mf_sync_items_per_page'] )
        ? $pre_api_options['mf_sync_items_per_page']
        : 20;
    $mf_shipping_product_id = isset( $pre_api_options['mf_shipping_product_id'] )
        ? $pre_api_options['mf_shipping_product_id']
        : '';
    $mf_part_refund_product_id = isset( $pre_api_options['mf_part_refund_product_id'] )
        ? $pre_api_options['mf_part_refund_product_id']
        : '';
    $pre_sync_product_tax_class = ( isset( $pre_api_options['is_tax_class_active'] ) )
        ? $pre_api_options['is_tax_class_active']
        : '';
    $pre_sync_product_categories = ( isset( $pre_api_options['is_product_categories_active'] ) )
        ? $pre_api_options['is_product_categories_active']
        : 'disabled';
    $pre_sync_campaigns = ( isset( $pre_api_options['is_campaigns_active'] ) )
        ? $pre_api_options['is_campaigns_active']
        : 'disabled';
    $pre_is_receipt_active = ( isset( $pre_api_options['is_receipt_active'] ) )
        ? $pre_api_options['is_receipt_active']
        : 'disabled';
    $pre_company_corp_no = ( isset( $pre_api_options['mf_company_corp_no'] ) )
        ? $pre_api_options['mf_company_corp_no']
        : '';
    $pre_company_name = ( isset( $pre_api_options['mf_company_name'] ) )
        ? $pre_api_options['mf_company_name']
        : '';
    $pre_bookkeeping_account_payments = ( isset( $pre_api_options['bookkeeping_account_payments'] ) && is_array( $pre_api_options['bookkeeping_account_payments'] ) )
        ? $pre_api_options['bookkeeping_account_payments']
        : [];
    $pre_bookkeeping_account_shipping = ( isset( $pre_api_options['mf_bookkeeping_account_shipping'] ) )
        ? $pre_api_options['mf_bookkeeping_account_shipping']
        : '';
    $pre_bookkeeping_account_tax = ( isset( $pre_api_options['mf_bookkeeping_account_tax'] ) )
        ? $pre_api_options['mf_bookkeeping_account_tax']
        : '';
} else {
    $pre_active_status = '';
    $pre_test_url = '';
    $pre_test_user = '';
    $pre_test_pw = '';
    $pre_production_url = '';
    $pre_production_user = '';
    $pre_production_pw = '';
    $mf_sync_status = '';
    $mf_sync_articles_interval = '';
    $pre_active_price_status = '';
    $sync_new_orders_to_apps = 'enabled';
    $sync_publish_products_status = 'disabled';
    $sync_only_product_stock_amount = 'disabled';
    $mf_shipping_product_id = '';
    $mf_part_refund_product_id = '';
    $pre_active_pic_status = '';
    $sync_article_description = 'disabled';
    $pre_active_product_title = '';
    $pre_sync_product_tax_class = 'enabled'; // Enabling tax rule button for first time installation
    $pre_sync_product_categories = 'disabled';
    $pre_sync_campaigns = 'disabled';
    $pre_sync_product_primary_category = 'disabled';
    $pre_is_receipt_active = 'disabled';
    $pre_company_corp_no = '';
    $pre_company_name = '';
    $pre_bookkeeping_account_payments = [];
    $pre_bookkeeping_account_shipping = '';
    $pre_bookkeeping_account_tax = '';
}

$payment_gateways = [];
if ( function_exists( 'WC' ) ) {
    foreach ( WC()->payment_gateways()->payment_gateways() as $gateway ) {
        if ( in_array( $gateway->id, [ 'cod' ] ) ) {
            continue;
        }
        $payment_gateways[ $gateway->id ] = $gateway->get_title();
    }
}
?>
<div class="wrap">
    <h1><?php _e('Moreflo Settings', 'wpmf'); ?></h1>
</div>

<?php if(!get_option('mf_account_ids')) { ?>
    <div class="error notice">
        <p><?php _e("Sorry, We're unable to connect with the Moreflo API. Please check your API settings and update.", 'wpmf');?></p>
    </div>
<?php } else { ?>
    <div class="updated  notice">
        <p><?php _e("Successfully connected to the Moreflo.", 'wpmf');?></p>
    </div>
<?php } ?>

<form method="post" action=''>
    <table class="form-table">
        <tr>
            <th colspan="2"><h2 class="title"><?php esc_html_e( 'Connection', 'wpmf' ) ?></h2></th>
        </tr>
        <tr>
            <th scope="row"><?php _e('Environment', 'wpmf'); ?></th>
            <td>
                <select name="url_select" id="url_select">
                    <option value="test" <?php selected( $pre_active_status, 'test' ) ?>><?php _e('Test', 'wpmf'); ?></option>
                    <option value="production" <?php selected( $pre_active_status, 'production' ) ?>><?php _e('Production', 'wpmf'); ?></option>
                </select>
            </td>
        </tr>
        <tr>
            <th scope="row"><?php _e('Test URL', 'wpmf'); ?></th>
            <td>
                <input type="url" name="api_test_url" class="regular-text code" value="<?php esc_attr_e( $pre_test_url ) ?>">
                <p class="description"><?php _e('Enter test URL like, https://test.api.moreflo.com/', 'wpmf'); ?></p>
            </td>
        </tr>
        <tr>
            <th scope="row"><?php _e('Test Username', 'wpmf'); ?></th>
            <td>
                <input type="text" name="api_test_user" class="regular-text code" value="<?php esc_attr_e( $pre_test_user ) ?>">
            </td>
        </tr>
        <tr>
            <th scope="row"><?php _e('Test Password', 'wpmf'); ?></th>
            <td>
                <input type="password" name="api_test_pw" class="regular-text code" value="<?php esc_attr_e( $pre_test_pw ) ?>">
            </td>
        </tr>
        <tr>
            <th scope="row"><?php esc_html_e('Production URL', 'wpmf'); ?></th>
            <td>
                <input type="url" name="api_production_url" class="regular-text code" value="<?php esc_attr_e( $pre_production_url ) ?>">
                <p class="description"><?php _e('Enter production URL like, https://api.moreflo.com/', 'wpmf'); ?></p>
            </td>
        </tr>
        <tr>
            <th scope="row"><?php _e('Production Username', 'wpmf'); ?></th>
            <td>
                <input type="text" name="api_production_user" class="regular-text code" value="<?php esc_attr_e( $pre_production_user ) ?>">
            </td>
        </tr>
        <tr>
            <th scope="row"><?php _e('Production Password', 'wpmf'); ?></th>
            <td>
                <input type="password" name="api_production_pw" class="regular-text code" value="<?php esc_attr_e( $pre_production_pw ) ?>">
            </td>
        </tr>
        <tr>
            <th colspan="2"><h2 class="title"><?php esc_html_e( 'Sync options', 'wpmf' ) ?></h2></th>
        </tr>
        <tr>
            <th scope="row"><?php _e('Automatic sync', 'wpmf'); ?></th>
            <td>
                <select name="mf_sync_status" id="mf_sync_status">
                    <?php
                    $sync_status = array(
                        'enabled' => __('Enabled', 'wpmf'),
                        'disabled' => __('Disabled', 'wpmf')
                    );

                    foreach ( $sync_status as $key => $value ) {
                        $selected = selected( $mf_sync_status, $key );
                        echo '<option value="'. esc_attr( $key ) .'" '. $selected .'>'. esc_html( $value ) .'</option>';
                    }
                    ?>
                </select>
            </td>
        </tr>
        <tr>
            <th scope="row"><?php echo esc_html__('Synchronize new orders to Apps', 'wpmf'); ?></th>
            <td>
                <select name="sync_new_orders_to_apps" id="sync_new_orders_to_apps">
                    <option value="enabled" <?php selected( $sync_new_orders_to_apps, 'enabled' ) ?>><?php echo esc_html__('Enabled', 'wpmf'); ?></option>
                    <option value="disabled" <?php selected( $sync_new_orders_to_apps, 'disabled' ) ?>><?php echo esc_html__('Disabled', 'wpmf'); ?></option>
                </select>
                <p class="description"><?php echo esc_html__('Select "Enabled" to sync all incomming orders to Apps.', 'wpmf') ?></p>
            </td>
        </tr>
        <tr>
            <th scope="row"><?php echo esc_html__('Synchronize publish status of products', 'wpmf'); ?></th>
            <td>
                <select name="sync_publish_products_status" id="sync_publish_products_status">
                    <option value="enabled" <?php selected( $sync_publish_products_status, 'enabled' ) ?>><?php echo esc_html__('Enabled', 'wpmf'); ?></option>
                    <option value="disabled" <?php selected( $sync_publish_products_status, 'disabled' ) ?>><?php echo esc_html__('Disabled', 'wpmf'); ?></option>
                </select>
                <p class="description">
                    <?php echo esc_html__('Select "Enabled" to publish new products when they are synced from Apps and set them to draft when deactivated in Apps.', 'wpmf'); ?><br />
                    <?php echo esc_html__('When "Disabled", products will be set as draft and you can publish them manually.', 'wpmf'); ?>
                </p>
            </td>
        </tr>
        <tr>
            <th scope="row"><?php echo esc_html__('Only sync product stock amount', 'wpmf'); ?></th>
            <td>
                <select name="sync_only_product_stock_amount" id="sync_only_product_stock_amount">
                    <option value="enabled" <?php selected( $sync_only_product_stock_amount, 'enabled' ) ?>><?php echo esc_html__('Enabled', 'wpmf'); ?></option>
                    <option value="disabled" <?php selected( $sync_only_product_stock_amount, 'disabled' ) ?>><?php echo esc_html__('Disabled', 'wpmf'); ?></option>
                </select>
                <p class="description">
                    <?php echo esc_html__('When "Enabled", only stock amount will be synced for products, no other product data. Also, no new products will be synced.', 'wpmf'); ?><br />
                </p>
            </td>
        </tr>
        <tr>
            <th scope="row"><?php echo esc_html__('Synchronize Price', 'wpmf'); ?></th>
            <td>
                <select name="sync_price_select" id="sync_price_select">
                    <option value="enabled" <?php selected( $pre_active_price_status, 'enabled' ) ?>><?php echo esc_html__('Enabled', 'wpmf'); ?></option>
                    <option value="disabled" <?php selected( $pre_active_price_status, 'disabled' ) ?>><?php echo esc_html__('Disabled', 'wpmf'); ?></option>
                </select>
                <p class="description"><?php echo esc_html__('Select "Enabled" to sync prices', 'wpmf'); ?></p>
            </td>
        </tr>
        <tr>
            <th scope="row"><?php echo esc_html__('Synchronize Article Description', 'wpmf'); ?></th>
            <td>
                <select name="sync_article_description" id="sync_article_description">
                    <option value="enabled" <?php selected( $sync_article_description, 'enabled' ) ?>><?php echo esc_html__('Enabled', 'wpmf'); ?></option>
                    <option value="disabled" <?php selected( $sync_article_description, 'disabled' ) ?>><?php echo esc_html__('Disabled', 'wpmf'); ?></option>
                </select>
                <p class="description"><?php echo esc_html__('Select "Enabled" to sync article description from Apps to WooCommerce. Any current content or changes to the content in WooCommerce will be replaced.', 'wpmf'); ?></p>
            </td>
        </tr>
        <tr>
            <th scope="row"><?php echo esc_html__('Synchronize Pictures', 'wpmf'); ?></th>
            <td>
                <select name="sync_pic_select" id="sync_pic_select">
                    <option value="enabled" <?php selected( $pre_active_pic_status, 'enabled' ) ?>><?php echo esc_html__('Enabled', 'wpmf'); ?></option>
                    <option value="disabled" <?php selected( $pre_active_pic_status, 'disabled' ) ?>><?php echo esc_html__('Disabled', 'wpmf'); ?></option>
                </select>
                <p class="description"><?php echo esc_html__('Select "Enabled" to sync pictures', 'wpmf'); ?></p>
            </td>
        </tr>
        <tr>
            <th scope="row"><?php echo esc_html__('Synchronize Product Title', 'wpmf'); ?></th>
            <td>
                <select name="sync_product_title_select" id="sync_product_title_select">
                    <option value="enabled" <?php selected( $pre_active_product_title, 'enabled' ) ?>><?php echo esc_html__('Enabled', 'wpmf'); ?></option>
                    <option value="disabled" <?php selected( $pre_active_product_title, 'disabled' ) ?>><?php echo esc_html__('Disabled', 'wpmf'); ?></option>
                </select>
                <p class="description"><?php echo esc_html__('Select "Enabled" to sync product title', 'wpmf'); ?></p>
            </td>
        </tr>
        <tr>
            <th scope="row"><?php echo esc_html__('Synchronize tax classes', 'wpmf'); ?></th>
            <td>
                <select name="sync_product_tax_class" id="sync_product_tax_class">
                    <option value="enabled" <?php selected( $pre_sync_product_tax_class, 'enabled' ) ?>><?php echo esc_html__('Enabled', 'wpmf'); ?></option>
                    <option value="disabled" <?php selected( $pre_sync_product_tax_class, 'disabled' ) ?>><?php echo esc_html__('Disabled', 'wpmf'); ?></option>
                </select>
                <p class="description"><?php echo esc_html__('Select "Enabled" to sync tax classes', 'wpmf'); ?></p>
            </td>
        </tr>
        <tr>
            <th scope="row"><?php echo esc_html__('Synchronize Campaigns', 'wpmf'); ?></th>
            <td>
                <select name="sync_campaigns" id="sync_campaigns">
                    <option value="enabled" <?php selected( $pre_sync_campaigns, 'enabled' ) ?>><?php echo esc_html__('Enabled', 'wpmf'); ?></option>
                    <option value="disabled" <?php selected( $pre_sync_campaigns, 'disabled' ) ?>><?php echo esc_html__('Disabled', 'wpmf'); ?></option>
                </select>
                <p class="description"><?php echo esc_html__('Select "Enabled" to sync campaigns from Apps to WooCommerce and use them in this shop. This feature should not be used together with other discounting plugins.', 'wpmf'); ?></p>
            </td>
        </tr>
        <tr>
            <th scope="row"><?php echo esc_html__('Synchronize article group', 'wpmf'); ?></th>
            <td>
                <select name="sync_product_categories" id="sync_product_categories">
                    <option value="enabled" <?php selected( $pre_sync_product_categories, 'enabled' ) ?>><?php echo esc_html__('Enabled', 'wpmf'); ?></option>
                    <option value="disabled" <?php selected( $pre_sync_product_categories, 'disabled' ) ?>><?php echo esc_html__('Disabled', 'wpmf'); ?></option>
                </select>
                <p class="description"><?php echo esc_html__('Select "Enabled" to sync groups to WooCommerce as product categories. Also changes the category for synced products to the articles\' group.', 'wpmf'); ?></p>
            </td>
        </tr>
        <tr>
            <th scope="row"><?php echo esc_html__('Create receipts in Apps', 'wpmf'); ?></th>
            <td>
                <select name="sync_receipt_active" id="sync_receipt_active">
                    <option value="enabled" <?php selected( $pre_is_receipt_active, 'enabled' ) ?>><?php echo esc_html__('Enabled', 'wpmf'); ?></option>
                    <option value="disabled" <?php selected( $pre_is_receipt_active, 'disabled' ) ?>><?php echo esc_html__('Disabled', 'wpmf'); ?></option>
                </select>
                <p class="description"><?php echo esc_html__('Select "Enabled" to create receipts for new orders. Please note: Number of decimals must be set to 2 in WooCommerce.', 'wpmf') ?></p>
            </td>
        </tr>
        <tr>
            <th scope="row"><?php echo esc_html__('Syncing Interval', 'wpmf'); ?></th>
            <td>
                <select name="mf_sync_articles_interval" id="mf_sync_articles_interval">
                    <?php
                    $schedules = wp_get_schedules();

                    if ( is_array( $schedules ) ) {
                        foreach ( $schedules as $key => $value ) {
                            $selected = selected( $mf_sync_articles_interval, $key, false );
                            echo '<option value="'. esc_attr( $key ) .'" '. $selected .'>'. esc_html( $value['display'] ) .'</option>';
                        }
                    }
                    ?>
                </select>                
            </td>
        </tr> 
        <tr>
            <th scope="row"><?php _e('No of articles to fetch per page', 'wpmf'); ?></th>
            <td>
                <select name="mf_sync_items_per_page" id="mf_sync_items_per_page">
                    <?php
                    $sync_status = array(
                        10 => 10,
                        20 => 20,
                        30 => 30,
                        50 => 50,
                        100 => 100,
                        200 => 200
                    );

                    if ( is_array( $sync_status ) ) {
                        foreach ( $sync_status as $key => $value ) {
                            echo '<option value="'. esc_attr( $key ) .'" '. selected( $mf_sync_items_per_page, $key ) .'>'. esc_html( $value ) .'</option>';
                        }
                    }
                    ?>
                </select>
            </td>
        </tr>
        <tr>
            <th colspan="2">
                <h2 class="title"><?php esc_html_e( 'Receipts', 'wpmf' ) ?></h2>
                <p><?= esc_html__( 'Only enter bookkeeping accounts for the payment options that should be booked automatically on new order.', 'wpmf' ) ?></p>
            </th>
        </tr>
        <tr>
            <th scope="row"><?php _e('Company Corp number', 'wpmf'); ?></th>
            <td>
                <input type="text" name="mf_company_corp_no" class="regular-text code" value="<?php esc_attr_e( $pre_company_corp_no ) ?>">
            </td>
        </tr>
        <tr>
            <th scope="row"><?php _e('Company name', 'wpmf'); ?></th>
            <td>
                <input type="text" name="mf_company_name" class="regular-text code" value="<?php esc_attr_e( $pre_company_name ) ?>">
            </td>
        </tr>
        <?php
        if ( ! empty( $payment_gateways ) ) { ?>
            <tr>
                <th scope="row"><?php esc_html_e('Bookkeeping account for payments', 'wpmf') ?></th>
                <td>
                    <?php
                    foreach ( $payment_gateways as $gateway_id => $gateway_label ) {
                        $value = isset( $pre_bookkeeping_account_payments[ $gateway_id ] )
                            ? $pre_bookkeeping_account_payments[ $gateway_id ]
                            : '';
                        ?>
                        <p>
                            <label><strong><?php esc_html_e( $gateway_label ) ?></strong></label><br />
                            <input
                                type="text"
                                name="mf_bookkeeping_account_payments_<?php esc_attr_e( $gateway_id ) ?>"
                                class="regular-text code"
                                value="<?php esc_attr_e( $value ) ?>"
                                />
                        </p>
                        <?php
                    }
                    ?>
                </td>
            </tr>
            <?php
        }
        ?>
        <tr>
            <th scope="row"><?php _e('Bookkeeping account for shipping', 'wpmf'); ?></th>
            <td>
                <input type="text" name="mf_bookkeeping_account_shipping" class="regular-text code" value="<?php esc_attr_e( $pre_bookkeeping_account_shipping ) ?>">
            </td>
        </tr>
        <tr>
            <th scope="row"><?php _e('Custom bookkeeping account for taxes', 'wpmf'); ?></th>
            <td>
                <input type="text" name="mf_bookkeeping_account_tax" class="regular-text code" value="<?php esc_attr_e( $pre_bookkeeping_account_tax ) ?>" disabled />
                <p class="description"><?= esc_html__( 'This setting will override the vat account settings made in Apps.', 'wpmf' ) ?> <strong><?= esc_html__( 'Deprecated: Will be removed soon. Please move it to Apps.', 'wpmf' ) ?></strong></p>
            </td>
        </tr>
        <?php
        if ( 'enabled' == $pre_is_receipt_active ) { ?>
            <tr>
                <th scope="row"><?php _e('Receipt vat accounts', 'wpmf'); ?></th>
                <td>
                    <?php
                    $vat_accounts = array_filter([
                        0 => MF_Sync_Bookkeeping::get_vat_account( 0 ),
                        6 => MF_Sync_Bookkeeping::get_vat_account( 6 ),
                        12 => MF_Sync_Bookkeeping::get_vat_account( 12 ),
                        25 => MF_Sync_Bookkeeping::get_vat_account( 25 ),
                    ]);
                    if ( empty( $vat_accounts ) ) { ?>
                        <span><?= esc_html__( 'Vat account have not been synced to WordPress. Please enter them in Apps and manually perform a Force Sync or wait for the next sync.', 'wpmf' ) ?></span>
                        <?php
                    } else { ?>
                        <fieldset>
                            <?php
                            $o = [];
                            foreach ( $vat_accounts as $vat_rate => $vat_account ) {
                                $o[] = '<label style="min-width: 2em;">'. intval( $vat_rate ) .'%</label><input type="text" value="'. intval( $vat_account ) .'" disabled />';
                            }
                            echo implode( '<br />', $o );
                            ?>
                        </fieldset>
                        <p class="description"><?= esc_html__('These settings can be changed in Apps.', 'wpmf') ?></p>
                        <?php
                    }
                    ?>
                </td>
            </tr>
            <?php
        }
        ?>
        <tr>
            <th colspan="2"><h2 class="title"><?php esc_html_e( 'Miscellaneous', 'wpmf' ) ?></h2></th>
        </tr>
        <tr>
            <th scope="row"><?php _e('Shipping Product Number', 'wpmf'); ?></th>
            <td>
                <input type="text" name="mf_shipping_product_id" class="regular-text code" value="<?php esc_attr_e( $mf_shipping_product_id ) ?>">
                <p class="description"><?php echo esc_html__('This article is used for shipping and should be set in Apps with "Set price on sale".', 'wpmf') ?></p>
            </td>
        </tr>
        <tr>
            <th scope="row"><?php _e('Part Refund Product Number', 'wpmf'); ?></th>
            <td>
                <input type="text" name="mf_part_refund_product_id" class="regular-text code" value="<?php esc_attr_e( $mf_part_refund_product_id ) ?>">
                <p class="description"><?php echo esc_html__('This article is used for refunds where quantity is zero. Must be set in Apps with "Set price on sale".', 'wpmf') ?></p>
            </td>
        </tr>
        <tr>
            <th scope="row"><?php _e('Enable the Debugging Logs', 'wpmf'); ?></th>
            <td>
                <input type="checkbox" name="mf_error_log" value="checked" <?php checked( $pre_api_options['log_enabled'], 'checked' ) ?> />
            </td>
        </tr>
    </table>

    <p class="submit"><input type="submit" name="add_api_url" class="button button-primary" value="Save Options"></p>
</form>
